import itertools
from collections.abc import Callable
from typing import Any

import nox

nox.options.reuse_existing_virtualenvs = True
nox.options.error_on_external_run = True
nox.options.default_venv_backend = "uv"

PYTHON_VERSIONS = ["3.14", "3.13", "3.12", "3.11", "3.10"]

GQL_CORE_VERSIONS = [
    "3.2.6",
    "3.3.0a12",
]

COMMON_PYTEST_OPTIONS = [
    "--cov=.",
    "--cov-append",
    "-n",
    "auto",
    "--showlocals",
    "-vv",
    "--ignore=tests/typecheckers",
    "--ignore=tests/cli",
    "--ignore=tests/benchmarks",
    "--ignore=tests/experimental/pydantic",
]

INTEGRATIONS = [
    "asgi",
    "aiohttp",
    "chalice",
    "channels",
    "django",
    "fastapi",
    "flask",
    "quart",
    "sanic",
    "litestar",
    "pydantic",
]


def _install_gql_core(session: nox.Session, version: str) -> None:
    session.install(f"graphql-core=={version}")


gql_core_parametrize = nox.parametrize(
    "gql_core",
    GQL_CORE_VERSIONS,
)


def with_gql_core_parametrize(name: str, params: list[str]) -> Callable[[Any], Any]:
    # github cache doesn't support comma in the name, this is a workaround.
    arg_names = f"{name}, gql_core"
    combinations = list(itertools.product(params, GQL_CORE_VERSIONS))
    ids = [f"{name}-{comb[0]}__graphql-core-{comb[1]}" for comb in combinations]
    return nox.parametrize(arg_names, combinations, ids=ids)


@nox.session(python=PYTHON_VERSIONS, name="Tests", tags=["tests"])
@gql_core_parametrize
def tests(session: nox.Session, gql_core: str) -> None:
    session.run_install(
        "uv",
        "sync",
        "--no-group=integrations",
        env={"UV_PROJECT_ENVIRONMENT": session.virtualenv.location},
    )
    _install_gql_core(session, gql_core)
    markers = (
        ["-m", f"not {integration}", f"--ignore=tests/{integration}"]
        for integration in INTEGRATIONS
    )
    markers = [item for sublist in markers for item in sublist]

    session.run(
        "pytest",
        *COMMON_PYTEST_OPTIONS,
        *markers,
    )


@nox.session(python=["3.12"], name="Django tests", tags=["tests"])
@with_gql_core_parametrize("django", ["5.1.3", "5.0.9", "4.2.0"])
def tests_django(session: nox.Session, django: str, gql_core: str) -> None:
    session.run_install(
        "uv",
        "sync",
        "--no-group=integrations",
        env={"UV_PROJECT_ENVIRONMENT": session.virtualenv.location},
    )
    _install_gql_core(session, gql_core)
    session.install(f"django~={django}")
    session.install("pytest-django")

    session.run("pytest", *COMMON_PYTEST_OPTIONS, "-m", "django")


@nox.session(python=["3.11"], name="Starlette tests", tags=["tests"])
@gql_core_parametrize
def tests_starlette(session: nox.Session, gql_core: str) -> None:
    session.run_install(
        "uv",
        "sync",
        "--no-group=integrations",
        env={"UV_PROJECT_ENVIRONMENT": session.virtualenv.location},
    )
    session.install("starlette")
    _install_gql_core(session, gql_core)
    session.run("pytest", *COMMON_PYTEST_OPTIONS, "-m", "asgi")


@nox.session(python=["3.11"], name="Test integrations", tags=["tests"])
@with_gql_core_parametrize(
    "integration",
    [
        "aiohttp",
        "chalice",
        "channels",
        "fastapi",
        "flask",
        "quart",
        "sanic",
        "litestar",
    ],
)
def tests_integrations(session: nox.Session, integration: str, gql_core: str) -> None:
    session.run_install(
        "uv",
        "sync",
        "--no-group=integrations",
        env={"UV_PROJECT_ENVIRONMENT": session.virtualenv.location},
    )
    session.install(integration)
    _install_gql_core(session, gql_core)
    if integration == "aiohttp":
        session.install("pytest-aiohttp")
    elif integration == "channels":
        session.install("pytest-django")
        session.install("daphne")

    session.run("pytest", *COMMON_PYTEST_OPTIONS, "-m", integration)


@nox.session(
    python=["3.10", "3.11", "3.12", "3.13"],
    name="Pydantic V1 tests",
    tags=["tests", "pydantic"],
)
@gql_core_parametrize
def test_pydantic(session: nox.Session, gql_core: str) -> None:
    session.run_install(
        "uv",
        "sync",
        "--no-group=integrations",
        env={"UV_PROJECT_ENVIRONMENT": session.virtualenv.location},
    )
    session.install("pydantic~=1.10")
    _install_gql_core(session, gql_core)
    session.run(
        "pytest",
        "--cov=.",
        "--cov-append",
        "-m",
        "pydantic",
        "--ignore=tests/cli",
        "--ignore=tests/benchmarks",
    )


@nox.session(python=PYTHON_VERSIONS, name="Pydantic tests", tags=["tests", "pydantic"])
@gql_core_parametrize
def test_pydantic_v2(session: nox.Session, gql_core: str) -> None:
    session.run_install(
        "uv",
        "sync",
        "--no-group=integrations",
        env={"UV_PROJECT_ENVIRONMENT": session.virtualenv.location},
    )
    session.install("pydantic>=2.2")
    _install_gql_core(session, gql_core)
    session.run(
        "pytest",
        "--cov=.",
        "--cov-append",
        "-m",
        "pydantic",
        "--ignore=tests/cli",
        "--ignore=tests/benchmarks",
    )


@nox.session(python=PYTHON_VERSIONS, name="Type checkers tests", tags=["tests"])
def tests_typecheckers(session: nox.Session) -> None:
    session.run_install(
        "uv",
        "sync",
        env={"UV_PROJECT_ENVIRONMENT": session.virtualenv.location},
    )
    session.install("pyright")
    session.install("pydantic")
    session.install("mypy")
    session.install("ty")

    session.run(
        "pytest",
        "--cov=.",
        "--cov-append",
        "tests/typecheckers",
        "-vv",
    )


@nox.session(python=PYTHON_VERSIONS, name="CLI tests", tags=["tests"])
def tests_cli(session: nox.Session) -> None:
    session.run_install(
        "uv",
        "sync",
        "--no-group=integrations",
        env={"UV_PROJECT_ENVIRONMENT": session.virtualenv.location},
    )
    session.install("uvicorn")
    session.install("starlette")

    session.run(
        "pytest",
        "--cov=.",
        "--cov-append",
        "tests/cli",
        "-vv",
    )



from __future__ import annotations

import json
import os
import pathlib
import subprocess
import tempfile
from typing import TypedDict

from .result import Result


class PyrightCLIResult(TypedDict):
    version: str
    time: str
    generalDiagnostics: list[GeneralDiagnostic]
    summary: Summary


class GeneralDiagnostic(TypedDict):
    file: str
    severity: str
    message: str
    range: Range


class Range(TypedDict):
    start: EndOrStart
    end: EndOrStart


class EndOrStart(TypedDict):
    line: int
    character: int


class Summary(TypedDict):
    filesAnalyzed: int
    errorCount: int
    warningCount: int
    informationCount: int
    timeInSec: float


def run_mypy(
    code: str, strict: bool = True, mypy_plugins: list[str] | None = None
) -> list[Result]:
    args = ["mypy", "--output=json"]

    if strict:
        args.append("--strict")

    with tempfile.TemporaryDirectory() as directory:
        module_path = pathlib.Path(directory) / "mypy_test.py"
        module_path.write_text(code)

        config_lines = ["[mypy]\n"]
        if mypy_plugins:
            config_lines.append(f"plugins = {', '.join(mypy_plugins)}\n")
        config_path = pathlib.Path(directory) / "mypy.ini"
        config_path.write_text("".join(config_lines))

        process_result = subprocess.run(
            [*args, "--config-file", str(config_path), str(module_path)],
            check=False,
            capture_output=True,
            env={
                "PYTHONWARNINGS": "error,ignore::SyntaxWarning",
                "PATH": os.environ["PATH"],
            },
        )

        full_output = (
            process_result.stdout.decode("utf-8")
            + "\n"
            + process_result.stderr.decode("utf-8")
        )
        full_output = full_output.strip()

        results: list[Result] = []

        try:
            for line in full_output.split("\n"):
                mypy_result = json.loads(line)

                results.append(
                    Result(
                        type=mypy_result["severity"].strip(),
                        message=mypy_result["message"].strip(),
                        line=mypy_result["line"],
                        column=mypy_result["column"] + 1,
                    )
                )
        except json.JSONDecodeError as e:
            raise ValueError(f"Invalid JSON: {full_output}") from e

        results.sort(key=lambda x: (x.line, x.column, x.message))

        return results



import asyncio
import contextlib
from collections.abc import AsyncGenerator
from enum import Enum
from typing import Any

from graphql import GraphQLError
from graphql.version import VersionInfo, version_info

import strawberry
from strawberry.extensions import SchemaExtension
from strawberry.file_uploads import Upload
from strawberry.permission import BasePermission
from strawberry.schema.config import StrawberryConfig
from strawberry.subscriptions.protocols.graphql_transport_ws.types import PingMessage
from strawberry.types import ExecutionContext


class AlwaysFailPermission(BasePermission):
    message = "You are not authorized"

    def has_permission(self, source: Any, info: strawberry.Info, **kwargs: Any) -> bool:
        return False


class MyExtension(SchemaExtension):
    def get_results(self) -> dict[str, str]:
        return {"example": "example"}


def _read_file(text_file: Upload) -> str:
    with contextlib.suppress(ModuleNotFoundError):
        from starlette.datastructures import UploadFile

        # allow to keep this function synchronous, starlette's files have
        # async methods for reading
        if isinstance(text_file, UploadFile):
            text_file = text_file.file._file  # type: ignore

    with contextlib.suppress(ModuleNotFoundError):
        from litestar.datastructures import UploadFile as LitestarUploadFile

        if isinstance(text_file, LitestarUploadFile):
            text_file = text_file.file  # type: ignore

    with contextlib.suppress(ModuleNotFoundError):
        from sanic.request import File as SanicUploadFile

        if isinstance(text_file, SanicUploadFile):
            return text_file.body.decode()

    return text_file.read().decode()


@strawberry.enum
class Flavor(Enum):
    VANILLA = "vanilla"
    STRAWBERRY = "strawberry"
    CHOCOLATE = "chocolate"


@strawberry.input
class FolderInput:
    files: list[Upload]


@strawberry.type
class DebugInfo:
    num_active_result_handlers: int
    is_connection_init_timeout_task_done: bool | None


@strawberry.type
class Hero:
    id: strawberry.ID

    @strawberry.field
    @staticmethod
    def name(fail: bool = False) -> str:
        if fail:
            raise ValueError("Failed to get name")

        return "Thiago Bellini"


@strawberry.type
class Query:
    @strawberry.field
    def greetings(self) -> str:
        return "hello"

    @strawberry.field
    def hello(self, name: str | None = None) -> str:
        return f"Hello {name or 'world'}"

    @strawberry.field
    async def async_hello(self, name: str | None = None, delay: float = 0) -> str:
        await asyncio.sleep(delay)
        return f"Hello {name or 'world'}"

    @strawberry.field(permission_classes=[AlwaysFailPermission])
    def always_fail(self) -> str | None:
        return "Hey"

    @strawberry.field
    async def error(self, message: str) -> AsyncGenerator[str, None]:
        yield GraphQLError(message)  # type: ignore

    @strawberry.field
    async def exception(self, message: str) -> str:
        raise ValueError(message)

    @strawberry.field
    async def some_error(self) -> str | None:
        raise ValueError("Some error")

    @strawberry.field
    def teapot(self, info: strawberry.Info[Any, None]) -> str:
        info.context["response"].status_code = 418

        return "🫖"

    @strawberry.field
    def root_name(self) -> str:
        return type(self).__name__

    @strawberry.field
    def value_from_context(
        self, info: strawberry.Info, key: str = "custom_value"
    ) -> str:
        return info.context[key]

    @strawberry.field
    def value_from_extensions(self, info: strawberry.Info, key: str) -> str:
        return info.input_extensions[key]

    @strawberry.field
    def returns_401(self, info: strawberry.Info) -> str:
        response = info.context["response"]
        if hasattr(response, "set_status"):
            response.set_status(401)
        else:
            response.status_code = 401

        return "hey"

    @strawberry.field
    def set_header(self, info: strawberry.Info, name: str) -> str:
        response = info.context["response"]
        response.headers["X-Name"] = name

        return name

    @strawberry.field
    def character(self) -> Hero:
        return Hero(id=strawberry.ID("1"))

    @strawberry.field
    async def streamable_field(self) -> strawberry.Streamable[str]:
        for i in range(2):
            yield f"Hello {i}"
            await asyncio.sleep(0.1)


@strawberry.type
class Mutation:
    @strawberry.mutation
    def echo(self, string_to_echo: str) -> str:
        return string_to_echo

    @strawberry.mutation
    def hello(self) -> str:
        return "strawberry"

    @strawberry.mutation
    def read_text(self, text_file: Upload) -> str:
        return _read_file(text_file)

    @strawberry.mutation
    def read_files(self, files: list[Upload]) -> list[str]:
        return list(map(_read_file, files))

    @strawberry.mutation
    def read_folder(self, folder: FolderInput) -> list[str]:
        return list(map(_read_file, folder.files))

    @strawberry.mutation
    def match_text(self, text_file: Upload, pattern: str) -> str:
        text = text_file.read().decode()
        return pattern if pattern in text else ""

    @strawberry.mutation
    def update_context(self, info: strawberry.Info, key: str, value: str) -> bool:
        info.context[key] = value
        return True


@strawberry.type
class Subscription:
    active_infinity_subscriptions = 0

    @strawberry.subscription
    async def echo(self, message: str, delay: float = 0) -> AsyncGenerator[str, None]:
        await asyncio.sleep(delay)
        yield message

    @strawberry.subscription
    async def request_ping(self, info: strawberry.Info) -> AsyncGenerator[bool, None]:
        ws = info.context["ws"]
        await ws.send_json(PingMessage({"type": "ping"}))
        yield True

    @strawberry.subscription
    async def infinity(self, message: str) -> AsyncGenerator[str, None]:
        Subscription.active_infinity_subscriptions += 1

        try:
            while True:
                yield message
                await asyncio.sleep(1)
        finally:
            Subscription.active_infinity_subscriptions -= 1

    @strawberry.subscription
    async def context(self, info: strawberry.Info) -> AsyncGenerator[str, None]:
        yield info.context["custom_value"]

    @strawberry.subscription
    async def error(self, message: str) -> AsyncGenerator[str, None]:
        yield GraphQLError(message)  # type: ignore

    @strawberry.subscription
    async def exception(self, message: str) -> AsyncGenerator[str, None]:
        raise ValueError(message)

        # Without this yield, the method is not recognised as an async generator
        yield "Hi"

    @strawberry.subscription
    async def flavors(self) -> AsyncGenerator[Flavor, None]:
        yield Flavor.VANILLA
        yield Flavor.STRAWBERRY
        yield Flavor.CHOCOLATE

    @strawberry.subscription
    async def flavors_invalid(self) -> AsyncGenerator[Flavor, None]:
        yield Flavor.VANILLA
        yield "invalid type"  # type: ignore
        yield Flavor.CHOCOLATE

    @strawberry.subscription
    async def debug(self, info: strawberry.Info) -> AsyncGenerator[DebugInfo, None]:
        active_result_handlers = [
            task for task in info.context["get_tasks"]() if not task.done()
        ]

        connection_init_timeout_task = info.context["connectionInitTimeoutTask"]
        is_connection_init_timeout_task_done = (
            connection_init_timeout_task.done()
            if connection_init_timeout_task
            else None
        )

        yield DebugInfo(
            num_active_result_handlers=len(active_result_handlers),
            is_connection_init_timeout_task_done=is_connection_init_timeout_task_done,
        )

    @strawberry.subscription
    async def listener(
        self,
        info: strawberry.Info,
        timeout: float | None = None,
        group: str | None = None,
    ) -> AsyncGenerator[str, None]:
        yield info.context["request"].channel_name

        async with info.context["request"].listen_to_channel(
            type="test.message",
            timeout=timeout,
            groups=[group] if group is not None else [],
        ) as cm:
            async for message in cm:
                yield message["text"]

    @strawberry.subscription
    async def listener_with_confirmation(
        self,
        info: strawberry.Info,
        timeout: float | None = None,
        group: str | None = None,
    ) -> AsyncGenerator[str | None, None]:
        async with info.context["request"].listen_to_channel(
            type="test.message",
            timeout=timeout,
            groups=[group] if group is not None else [],
        ) as cm:
            yield None
            yield info.context["request"].channel_name
            async for message in cm:
                yield message["text"]

    @strawberry.subscription
    async def connection_params(
        self, info: strawberry.Info
    ) -> AsyncGenerator[strawberry.scalars.JSON, None]:
        yield info.context["connection_params"]

    @strawberry.subscription
    async def long_finalizer(
        self, info: strawberry.Info, delay: float = 0
    ) -> AsyncGenerator[str, None]:
        try:
            for _i in range(100):
                yield "hello"
                await asyncio.sleep(0.01)
        finally:
            await asyncio.sleep(delay)


class Schema(strawberry.Schema):
    def process_errors(
        self, errors: list, execution_context: ExecutionContext | None = None
    ) -> None:
        import traceback

        traceback.print_stack()
        return super().process_errors(errors, execution_context)


schema = Schema(
    query=Query,
    mutation=Mutation,
    subscription=Subscription,
    extensions=[MyExtension],
    config=StrawberryConfig(
        enable_experimental_incremental_execution=(
            version_info >= VersionInfo.from_str("3.3.0a0")
        )
    ),
)



from __future__ import annotations

import asyncio
import contextvars
import functools
import json
import urllib.parse
from io import BytesIO
from typing import Any, Literal

from flask import Flask
from flask import Request as FlaskRequest
from flask import Response as FlaskResponse
from strawberry.flask.views import GraphQLView as BaseGraphQLView
from strawberry.http import GraphQLHTTPResponse
from strawberry.http.ides import GraphQL_IDE
from strawberry.schema import Schema
from strawberry.types import ExecutionResult
from tests.http.context import get_context
from tests.views.schema import Query

from .base import JSON, HttpClient, Response, ResultOverrideFunction


class GraphQLView(BaseGraphQLView[dict[str, object], object]):
    # this allows to test our code path for checking the request type
    # TODO: we might want to remove our check since it is done by flask
    # already
    methods = ["GET", "POST", "PUT", "PATCH", "DELETE", "HEAD"]

    result_override: ResultOverrideFunction = None

    def __init__(self, *args: Any, **kwargs: Any):
        self.result_override = kwargs.pop("result_override")
        super().__init__(*args, **kwargs)

    def get_root_value(self, request: FlaskRequest) -> object:
        super().get_root_value(request)  # for coverage
        return Query()

    def get_context(
        self, request: FlaskRequest, response: FlaskResponse
    ) -> dict[str, object]:
        context = super().get_context(request, response)

        return get_context(context)

    def process_result(
        self, request: FlaskRequest, result: ExecutionResult
    ) -> GraphQLHTTPResponse:
        if self.result_override:
            return self.result_override(result)

        return super().process_result(request, result)


class FlaskHttpClient(HttpClient):
    def __init__(
        self,
        schema: Schema,
        graphql_ide: GraphQL_IDE | None = "graphiql",
        allow_queries_via_get: bool = True,
        result_override: ResultOverrideFunction = None,
        multipart_uploads_enabled: bool = False,
    ):
        self.app = Flask(__name__)
        self.app.debug = True

        view = GraphQLView.as_view(
            "graphql_view",
            schema=schema,
            graphql_ide=graphql_ide,
            allow_queries_via_get=allow_queries_via_get,
            result_override=result_override,
            multipart_uploads_enabled=multipart_uploads_enabled,
        )

        self.app.add_url_rule(
            "/graphql",
            view_func=view,
        )

    async def _graphql_request(
        self,
        method: Literal["get", "post"],
        query: str | None = None,
        operation_name: str | None = None,
        variables: dict[str, object] | None = None,
        files: dict[str, BytesIO] | None = None,
        headers: dict[str, str] | None = None,
        extensions: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> Response:
        body = self._build_body(
            query=query,
            operation_name=operation_name,
            variables=variables,
            files=files,
            method=method,
            extensions=extensions,
        )

        data: dict[str, object] | str | None = None

        if body and files:
            body.update({name: (file, name) for name, file in files.items()})

        url = "/graphql"

        if method == "get":
            body_encoded = urllib.parse.urlencode(body or {})
            url = f"{url}?{body_encoded}"
        else:
            if body:
                data = body if files else json.dumps(body)
            kwargs["data"] = data

        headers = self._get_headers(method=method, headers=headers, files=files)

        return await self.request(url, method, headers=headers, **kwargs)

    def _do_request(
        self,
        url: str,
        method: Literal["get", "post", "patch", "put", "delete"],
        headers: dict[str, str] | None = None,
        **kwargs: Any,
    ):
        with self.app.test_client() as client:
            response = getattr(client, method)(url, headers=headers, **kwargs)

        return Response(
            status_code=response.status_code,
            data=response.data,
            headers=response.headers,
        )

    async def request(
        self,
        url: str,
        method: Literal["head", "get", "post", "patch", "put", "delete"],
        headers: dict[str, str] | None = None,
        **kwargs: Any,
    ) -> Response:
        loop = asyncio.get_running_loop()
        ctx = contextvars.copy_context()
        func_call = functools.partial(
            ctx.run, self._do_request, url=url, method=method, headers=headers, **kwargs
        )
        return await loop.run_in_executor(None, func_call)  # type: ignore

    async def get(
        self,
        url: str,
        headers: dict[str, str] | None = None,
    ) -> Response:
        return await self.request(url, "get", headers=headers)

    async def post(
        self,
        url: str,
        data: bytes | None = None,
        json: JSON | None = None,
        headers: dict[str, str] | None = None,
    ) -> Response:
        return await self.request(url, "post", headers=headers, data=data, json=json)



# ruff: noqa: F821
from __future__ import annotations

from collections import abc  # noqa: F401
from collections.abc import AsyncGenerator, AsyncIterable, AsyncIterator  # noqa: F401
from typing import (
    Annotated,
    Any,
)

import pytest

import strawberry
from strawberry.types.execution import PreExecutionError
from strawberry.utils.aio import aclosing


@pytest.mark.asyncio
async def test_subscription():
    @strawberry.type
    class Query:
        x: str = "Hello"

    @strawberry.type
    class Subscription:
        @strawberry.subscription
        async def example(self) -> AsyncGenerator[str, None]:
            yield "Hi"

    schema = strawberry.Schema(query=Query, subscription=Subscription)

    query = "subscription { example }"

    async with aclosing(await schema.subscribe(query)) as sub_result:
        result = await sub_result.__anext__()

    assert not result.errors
    assert result.data["example"] == "Hi"


@pytest.mark.asyncio
async def test_subscription_with_permission():
    from strawberry import BasePermission

    class IsAuthenticated(BasePermission):
        message = "Unauthorized"

        async def has_permission(
            self, source: Any, info: strawberry.Info, **kwargs: Any
        ) -> bool:
            return True

    @strawberry.type
    class Query:
        x: str = "Hello"

    @strawberry.type
    class Subscription:
        @strawberry.subscription(permission_classes=[IsAuthenticated])
        async def example(self) -> AsyncGenerator[str, None]:
            yield "Hi"

    schema = strawberry.Schema(query=Query, subscription=Subscription)

    query = "subscription { example }"

    async with aclosing(await schema.subscribe(query)) as sub_result:
        result = await sub_result.__anext__()

    assert not result.errors
    assert result.data["example"] == "Hi"


@pytest.mark.asyncio
async def test_subscription_with_arguments():
    @strawberry.type
    class Query:
        x: str = "Hello"

    @strawberry.type
    class Subscription:
        @strawberry.subscription
        async def example(self, name: str) -> AsyncGenerator[str, None]:
            yield f"Hi {name}"

    schema = strawberry.Schema(query=Query, subscription=Subscription)

    query = 'subscription { example(name: "Nina") }'

    async with aclosing(await schema.subscribe(query)) as sub_result:
        result = await sub_result.__anext__()

    assert not result.errors
    assert result.data["example"] == "Hi Nina"


@pytest.mark.parametrize(
    "return_annotation",
    [
        "AsyncGenerator[str, None]",
        "AsyncIterable[str]",
        "AsyncIterator[str]",
        "abc.AsyncIterator[str]",
        "abc.AsyncGenerator[str, None]",
        "abc.AsyncIterable[str]",
    ],
)
@pytest.mark.asyncio
async def test_subscription_return_annotations(return_annotation: str):
    async def async_resolver():
        yield "Hi"

    async_resolver.__annotations__["return"] = return_annotation

    @strawberry.type
    class Query:
        x: str = "Hello"

    @strawberry.type
    class Subscription:
        example = strawberry.subscription(resolver=async_resolver)

    schema = strawberry.Schema(query=Query, subscription=Subscription)

    query = "subscription { example }"

    async with aclosing(await schema.subscribe(query)) as sub_result:
        result = await sub_result.__anext__()

    assert not result.errors
    assert result.data["example"] == "Hi"


@pytest.mark.asyncio
async def test_subscription_with_unions():
    global A, B

    @strawberry.type
    class A:
        a: str

    @strawberry.type
    class B:
        b: str

    @strawberry.type
    class Query:
        x: str = "Hello"

    @strawberry.type
    class Subscription:
        @strawberry.subscription
        async def example_with_union(self) -> AsyncGenerator[A | B, None]:
            yield A(a="Hi")

    schema = strawberry.Schema(query=Query, subscription=Subscription)

    query = "subscription { exampleWithUnion { ... on A { a } } }"

    async with aclosing(await schema.subscribe(query)) as sub_result:
        result = await sub_result.__anext__()

    assert not result.errors
    assert result.data["exampleWithUnion"]["a"] == "Hi"

    del A, B


@pytest.mark.asyncio
async def test_subscription_with_unions_and_annotated():
    global C, D

    @strawberry.type
    class C:
        c: str

    @strawberry.type
    class D:
        d: str

    @strawberry.type
    class Query:
        x: str = "Hello"

    @strawberry.type
    class Subscription:
        @strawberry.subscription
        async def example_with_annotated_union(
            self,
        ) -> AsyncGenerator[Annotated[C | D, strawberry.union("UnionName")], None]:
            yield C(c="Hi")

    schema = strawberry.Schema(query=Query, subscription=Subscription)

    query = "subscription { exampleWithAnnotatedUnion { ... on C { c } } }"

    async with aclosing(await schema.subscribe(query)) as sub_result:
        result = await sub_result.__anext__()

    assert not result.errors
    assert result.data["exampleWithAnnotatedUnion"]["c"] == "Hi"

    del C, D


@pytest.mark.asyncio
async def test_subscription_with_annotated():
    @strawberry.type
    class Query:
        x: str = "Hello"

    @strawberry.type
    class Subscription:
        @strawberry.subscription
        async def example(
            self,
        ) -> Annotated[AsyncGenerator[str, None], "this doesn't matter"]:
            yield "Hi"

    schema = strawberry.Schema(query=Query, subscription=Subscription)

    query = "subscription { example }"

    async with aclosing(await schema.subscribe(query)) as sub_result:
        result = await sub_result.__anext__()

    assert not result.errors
    assert result.data["example"] == "Hi"


async def test_subscription_immediate_error():
    @strawberry.type
    class Query:
        x: str = "Hello"

    @strawberry.type
    class Subscription:
        @strawberry.subscription()
        async def example(self) -> AsyncGenerator[str, None]:
            return "fds"

    schema = strawberry.Schema(query=Query, subscription=Subscription)

    query = """#graphql
        subscription { example }
    """

    async with aclosing(await schema.subscribe(query)) as sub_result:
        result = await sub_result.__anext__()

    assert isinstance(result, PreExecutionError)
    assert result.errors


async def test_wrong_operation_variables():
    @strawberry.type
    class Query:
        x: str = "Hello"

    @strawberry.type
    class Subscription:
        @strawberry.subscription
        async def example(self, name: str) -> AsyncGenerator[str, None]:
            yield f"Hi {name}"  # pragma: no cover

    schema = strawberry.Schema(query=Query, subscription=Subscription)

    query = """#graphql
        subscription subOp($opVar: String!){ example(name: $opVar) }
    """

    async with aclosing(await schema.subscribe(query)) as sub_result:
        result = await sub_result.__anext__()

    assert result.errors
    assert (
        result.errors[0].message
        == "Variable '$opVar' of required type 'String!' was not provided."
    )



from enum import Enum
from typing import Annotated, NewType

import strawberry

ExampleScalar = strawberry.scalar(
    NewType("ExampleScalar", object),
    serialize=lambda v: v,
    parse_value=lambda v: v,
)


@strawberry.type
class A:
    name: str


@strawberry.type
class B:
    a: A


UnionExample = Annotated[A | B, strawberry.union("UnionExample")]


class SampleClass:
    def __init__(self, schema):
        self.schema = schema


@strawberry.enum
class Role(Enum):
    ADMIN = "ADMIN"
    USER = "USER"


@strawberry.type
class User:
    name: str
    age: int
    role: Role
    example_scalar: ExampleScalar
    union_example: UnionExample
    inline_union: Annotated[A | B, strawberry.union("InlineUnion")]


@strawberry.type
class Query:
    @strawberry.field
    def user(self) -> User:
        return User(name="Patrick", age=100)


def create_schema():
    return strawberry.Schema(query=Query)


schema = create_schema()
sample_instance = SampleClass(schema)
not_a_schema = 42



import textwrap

import strawberry
from strawberry.federation.schema_directives import Link
from tests.conftest import skip_if_gql_32


def test_link_directive():
    @strawberry.type
    class Query:
        hello: str

    schema = strawberry.federation.Schema(
        query=Query,
        schema_directives=[
            Link(
                url="https://specs.apollo.dev/link/v1.0",
            )
        ],
    )

    expected = """
        schema @link(url: "https://specs.apollo.dev/link/v1.0") {
          query: Query
        }

        type Query {
          _service: _Service!
          hello: String!
        }

        scalar _Any

        type _Service {
          sdl: String!
        }
    """

    assert schema.as_str() == textwrap.dedent(expected).strip()


@skip_if_gql_32("formatting is different in gql 3.2")
def test_link_directive_imports():
    @strawberry.type
    class Query:
        hello: str

    schema = strawberry.federation.Schema(
        query=Query,
        schema_directives=[
            Link(
                url="https://specs.apollo.dev/federation/v2.11",
                import_=[
                    "@key",
                    "@requires",
                    "@provides",
                    "@external",
                    {"name": "@tag", "as": "@mytag"},
                    "@extends",
                    "@shareable",
                    "@inaccessible",
                    "@override",
                ],
            )
        ],
    )

    expected = """
    schema @link(url: "https://specs.apollo.dev/federation/v2.11", import: [
      "@key"
      "@requires"
      "@provides"
      "@external"
      { name: "@tag", as: "@mytag" }
      "@extends"
      "@shareable"
      "@inaccessible"
      "@override"
    ]) {
      query: Query
    }

    type Query {
      _service: _Service!
      hello: String!
    }

    scalar _Any

    type _Service {
      sdl: String!
    }
    """

    assert schema.as_str() == textwrap.dedent(expected).strip()


def test_adds_link_directive_automatically():
    @strawberry.federation.type(keys=["id"])
    class User:
        id: strawberry.ID

    @strawberry.type
    class Query:
        user: User

    schema = strawberry.federation.Schema(query=Query)

    expected = """
        schema @link(url: "https://specs.apollo.dev/federation/v2.11", import: ["@key"]) {
          query: Query
        }

        type Query {
          _entities(representations: [_Any!]!): [_Entity]!
          _service: _Service!
          user: User!
        }

        type User @key(fields: "id") {
          id: ID!
        }

        scalar _Any

        union _Entity = User

        type _Service {
          sdl: String!
        }
    """

    assert schema.as_str() == textwrap.dedent(expected).strip()


def test_adds_link_directive_from_interface():
    @strawberry.federation.interface(keys=["id"])
    class SomeInterface:
        id: strawberry.ID

    @strawberry.type
    class User:
        id: strawberry.ID

    @strawberry.type
    class Query:
        user: User

    schema = strawberry.federation.Schema(query=Query, types=[SomeInterface])

    expected = """
        schema @link(url: "https://specs.apollo.dev/federation/v2.11", import: ["@key"]) {
          query: Query
        }

        type Query {
          _service: _Service!
          user: User!
        }

        interface SomeInterface @key(fields: "id") {
          id: ID!
        }

        type User {
          id: ID!
        }

        scalar _Any

        type _Service {
          sdl: String!
        }
    """

    assert schema.as_str() == textwrap.dedent(expected).strip()


def test_adds_link_directive_from_input_types():
    @strawberry.federation.input(inaccessible=True)
    class SomeInput:
        id: strawberry.ID

    @strawberry.type
    class User:
        id: strawberry.ID

    @strawberry.type
    class Query:
        user: User

    schema = strawberry.federation.Schema(query=Query, types=[SomeInput])

    expected = """
        schema @link(url: "https://specs.apollo.dev/federation/v2.11", import: ["@inaccessible"]) {
          query: Query
        }

        type Query {
          _service: _Service!
          user: User!
        }

        input SomeInput @inaccessible {
          id: ID!
        }

        type User {
          id: ID!
        }

        scalar _Any

        type _Service {
          sdl: String!
        }
    """

    assert schema.as_str() == textwrap.dedent(expected).strip()


def test_adds_link_directive_automatically_from_field():
    @strawberry.federation.type(keys=["id"])
    class User:
        id: strawberry.ID
        age: int = strawberry.federation.field(tags=["private"])

    @strawberry.type
    class Query:
        user: User

    schema = strawberry.federation.Schema(query=Query)

    expected = """
        schema @link(url: "https://specs.apollo.dev/federation/v2.11", import: ["@key", "@tag"]) {
          query: Query
        }

        type Query {
          _entities(representations: [_Any!]!): [_Entity]!
          _service: _Service!
          user: User!
        }

        type User @key(fields: "id") {
          id: ID!
          age: Int! @tag(name: "private")
        }

        scalar _Any

        union _Entity = User

        type _Service {
          sdl: String!
        }
    """

    assert schema.as_str() == textwrap.dedent(expected).strip()


def test_adds_directive_link_for_federation():
    @strawberry.federation.type(keys=["id"])
    class User:
        id: strawberry.ID

    @strawberry.type
    class Query:
        user: User

    schema = strawberry.federation.Schema(query=Query)

    expected = """
        schema @link(url: "https://specs.apollo.dev/federation/v2.11", import: ["@key"]) {
          query: Query
        }

        type Query {
          _entities(representations: [_Any!]!): [_Entity]!
          _service: _Service!
          user: User!
        }

        type User @key(fields: "id") {
          id: ID!
        }

        scalar _Any

        union _Entity = User

        type _Service {
          sdl: String!
        }
    """

    assert schema.as_str() == textwrap.dedent(expected).strip()


def test_adds_link_directive_automatically_from_scalar():
    # TODO: Federation scalar
    @strawberry.scalar
    class X:
        pass

    @strawberry.federation.type(keys=["id"])
    class User:
        id: strawberry.ID
        age: X

    @strawberry.type
    class Query:
        user: User

    schema = strawberry.federation.Schema(query=Query)

    expected = """
        schema @link(url: "https://specs.apollo.dev/federation/v2.11", import: ["@key"]) {
          query: Query
        }

        type Query {
          _entities(representations: [_Any!]!): [_Entity]!
          _service: _Service!
          user: User!
        }

        type User @key(fields: "id") {
          id: ID!
          age: X!
        }

        scalar X

        scalar _Any

        union _Entity = User

        type _Service {
          sdl: String!
        }
    """

    assert schema.as_str() == textwrap.dedent(expected).strip()



from inline_snapshot import snapshot
from typer import Typer
from typer.testing import CliRunner


def test_schema_export(cli_app: Typer, cli_runner: CliRunner):
    selector = "tests.fixtures.sample_package.sample_module:schema"
    result = cli_runner.invoke(cli_app, ["export-schema", selector])

    assert result.exit_code == 0
    assert result.stdout == snapshot(
        """\
type A {
  name: String!
}

type B {
  a: A!
}

scalar ExampleScalar

union InlineUnion = A | B

type Query {
  user: User!
}

enum Role {
  ADMIN
  USER
}

union UnionExample = A | B

type User {
  name: String!
  age: Int!
  role: Role!
  exampleScalar: ExampleScalar!
  unionExample: UnionExample!
  inlineUnion: InlineUnion!
}
"""
    )


def test_schema_symbol_is_callable(cli_app: Typer, cli_runner: CliRunner):
    selector = "tests.fixtures.sample_package.sample_module:create_schema"
    result = cli_runner.invoke(cli_app, ["export-schema", selector])

    assert result.exit_code == 0


def test_default_schema_symbol_name(cli_app: Typer, cli_runner: CliRunner):
    selector = "tests.fixtures.sample_package.sample_module"
    result = cli_runner.invoke(cli_app, ["export-schema", selector])

    assert result.exit_code == 0


def test_app_dir_option(cli_app: Typer, cli_runner: CliRunner):
    selector = "sample_module"
    result = cli_runner.invoke(
        cli_app,
        ["export-schema", "--app-dir=./tests/fixtures/sample_package", selector],
    )

    assert result.exit_code == 0


def test_invalid_module(cli_app: Typer, cli_runner: CliRunner):
    selector = "not.existing.module"
    result = cli_runner.invoke(cli_app, ["export-schema", selector])

    expected_error = "Error: No module named 'not'"

    assert result.exit_code == 2
    assert expected_error in result.stdout.replace("\n", "")


def test_invalid_symbol(cli_app: Typer, cli_runner: CliRunner):
    selector = "tests.fixtures.sample_package.sample_module:not.existing.symbol"
    result = cli_runner.invoke(cli_app, ["export-schema", selector])

    expected_error = (
        "Error: module 'tests.fixtures.sample_package.sample_module' "
        "has no attribute 'not'"
    )

    assert result.exit_code == 2
    assert expected_error in result.stdout.replace("\n", "")


def test_invalid_schema_instance(cli_app: Typer, cli_runner: CliRunner):
    selector = "tests.fixtures.sample_package.sample_module:not_a_schema"
    result = cli_runner.invoke(cli_app, ["export-schema", selector])

    expected_error = "Error: The `schema` must be an instance of strawberry.Schema"

    assert result.exit_code == 2
    assert expected_error in result.stdout.replace("\n", "")


def test_output_option(cli_app: Typer, cli_runner: CliRunner, tmp_path):
    selector = "tests.fixtures.sample_package.sample_module:schema"
    output = tmp_path / "schema.graphql"
    output_commands = ["--output", "-o"]
    for output_command in output_commands:
        result = cli_runner.invoke(
            cli_app, ["export-schema", selector, output_command, str(output)]
        )

        assert result.exit_code == 0
        assert output.read_text() == snapshot(
            """\
type A {
  name: String!
}

type B {
  a: A!
}

scalar ExampleScalar

union InlineUnion = A | B

type Query {
  user: User!
}

enum Role {
  ADMIN
  USER
}

union UnionExample = A | B

type User {
  name: String!
  age: Int!
  role: Role!
  exampleScalar: ExampleScalar!
  unionExample: UnionExample!
  inlineUnion: InlineUnion!
}
"""
        )



import pytest

import strawberry
from strawberry.schema.config import StrawberryConfig


def test_runs_parsing():
    @strawberry.type
    class Query:
        name: str

    schema = strawberry.Schema(query=Query)

    query = """
        query {
            example
    """

    result = schema.execute_sync(query)

    assert len(result.errors) == 1
    assert result.errors[0].message == "Syntax Error: Expected Name, found <EOF>."


@pytest.mark.asyncio
@pytest.mark.filterwarnings("ignore:.* was never awaited:RuntimeWarning")
async def test_errors_when_running_async_in_sync_mode():
    @strawberry.type
    class Query:
        @strawberry.field
        async def name(self) -> str:
            return "Patrick"

    schema = strawberry.Schema(query=Query)

    query = """
        query {
            name
        }
    """

    result = schema.execute_sync(query)
    assert len(result.errors) == 1
    assert isinstance(result.errors[0].original_error, RuntimeError)
    assert (
        result.errors[0].message
        == "GraphQL execution failed to complete synchronously."
    )


@pytest.mark.asyncio
async def test_runs_parsing_async():
    @strawberry.type
    class Query:
        example: str

    schema = strawberry.Schema(query=Query)

    query = """
        query {
            example
    """

    result = await schema.execute(query)

    assert len(result.errors) == 1
    assert result.errors[0].message == "Syntax Error: Expected Name, found <EOF>."


def test_suggests_fields_by_default():
    @strawberry.type
    class Query:
        name: str

    schema = strawberry.Schema(query=Query)

    query = """
        query {
            ample
        }
    """

    result = schema.execute_sync(query)

    assert len(result.errors) == 1
    assert (
        result.errors[0].message
        == "Cannot query field 'ample' on type 'Query'. Did you mean 'name'?"
    )


def test_can_disable_field_suggestions():
    @strawberry.type
    class Query:
        name: str

    schema = strawberry.Schema(
        query=Query, config=StrawberryConfig(disable_field_suggestions=True)
    )

    query = """
        query {
            ample
        }
    """

    result = schema.execute_sync(query)

    assert len(result.errors) == 1
    assert result.errors[0].message == "Cannot query field 'ample' on type 'Query'."


def test_can_disable_field_suggestions_multiple_fields():
    @strawberry.type
    class Query:
        name: str
        age: str

    schema = strawberry.Schema(
        query=Query, config=StrawberryConfig(disable_field_suggestions=True)
    )

    query = """
        query {
            ample
            ag
        }
    """

    result = schema.execute_sync(query)

    assert len(result.errors) == 2
    assert result.errors[0].message == "Cannot query field 'ample' on type 'Query'."
    assert result.errors[1].message == "Cannot query field 'ag' on type 'Query'."



from inline_snapshot import snapshot

from .utils.marks import requires_mypy, requires_pyright, requires_ty, skip_on_windows
from .utils.typecheck import Result, typecheck

pytestmark = [skip_on_windows, requires_pyright, requires_mypy, requires_ty]


def test_with_params():
    CODE = """
import strawberry

def example(info: strawberry.Info[None, None]) -> None:
    reveal_type(info.context)
    reveal_type(info.root_value)
"""

    results = typecheck(CODE)

    assert results.pyright == snapshot(
        [
            Result(
                type="information",
                message='Type of "info.context" is "None"',
                line=5,
                column=17,
            ),
            Result(
                type="information",
                message='Type of "info.root_value" is "None"',
                line=6,
                column=17,
            ),
        ]
    )
    assert results.mypy == snapshot(
        [
            Result(type="note", message='Revealed type is "None"', line=5, column=17),
            Result(type="note", message='Revealed type is "None"', line=6, column=17),
        ]
    )
    assert results.ty == snapshot(
        [
            Result(
                type="information",
                message="Revealed type: `None`",
                line=5,
                column=17,
            ),
            Result(
                type="information",
                message="Revealed type: `None`",
                line=6,
                column=17,
            ),
        ]
    )


def test_with_one_param():
    CODE = """
import strawberry

def example(info: strawberry.Info[None]) -> None:
    reveal_type(info.context)
    reveal_type(info.root_value)
"""

    results = typecheck(CODE)

    assert results.pyright == snapshot(
        [
            Result(
                type="information",
                message='Type of "info.context" is "None"',
                line=5,
                column=17,
            ),
            Result(
                type="information",
                message='Type of "info.root_value" is "Any"',
                line=6,
                column=17,
            ),
        ]
    )
    assert results.mypy == snapshot(
        [
            Result(type="note", message='Revealed type is "None"', line=5, column=17),
            Result(type="note", message='Revealed type is "Any"', line=6, column=17),
        ]
    )
    assert results.ty == snapshot(
        [
            Result(
                type="information",
                message="Revealed type: `None`",
                line=5,
                column=17,
            ),
            Result(
                type="information",
                message="Revealed type: `Any`",
                line=6,
                column=17,
            ),
        ]
    )


def test_without_params():
    CODE = """
import strawberry

def example(info: strawberry.Info) -> None:
    reveal_type(info.context)
    reveal_type(info.root_value)
"""

    results = typecheck(CODE)

    assert results.pyright == snapshot(
        [
            Result(
                type="information",
                message='Type of "info.context" is "Any"',
                line=5,
                column=17,
            ),
            Result(
                type="information",
                message='Type of "info.root_value" is "Any"',
                line=6,
                column=17,
            ),
        ]
    )
    assert results.mypy == snapshot(
        [
            Result(type="note", message='Revealed type is "Any"', line=5, column=17),
            Result(type="note", message='Revealed type is "Any"', line=6, column=17),
        ]
    )
    assert results.ty == snapshot(
        [
            Result(
                type="information",
                message="Revealed type: `Any`",
                line=5,
                column=17,
            ),
            Result(
                type="information",
                message="Revealed type: `Any`",
                line=6,
                column=17,
            ),
        ]
    )



from dataclasses import dataclass
from typing import Generic, TypeVar

import pytest

import strawberry
from strawberry.annotation import StrawberryAnnotation
from strawberry.exceptions import PrivateStrawberryFieldError
from strawberry.types.field import StrawberryField


def test_private_field():
    @strawberry.type
    class Query:
        name: str
        age: strawberry.Private[int]

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"
    assert len(definition.fields) == 1

    assert definition.fields[0].python_name == "name"
    assert definition.fields[0].graphql_name is None
    assert definition.fields[0].type is str

    instance = Query(name="Luke", age=22)
    assert instance.name == "Luke"
    assert instance.age == 22


@pytest.mark.raises_strawberry_exception(
    PrivateStrawberryFieldError,
    match=("Field age on type Query cannot be both private and a strawberry.field"),
)
def test_private_field_with_strawberry_field_error():
    @strawberry.type
    class Query:
        name: str
        age: strawberry.Private[int] = strawberry.field(description="🤫")


def test_private_field_access_in_resolver():
    @strawberry.type
    class Query:
        name: str
        age: strawberry.Private[int]

        @strawberry.field
        def age_in_months(self) -> int:
            return self.age * 12

    schema = strawberry.Schema(query=Query)

    result = schema.execute_sync(
        "query { ageInMonths }", root_value=Query(name="Dave", age=7)
    )

    assert not result.errors
    assert result.data == {
        "ageInMonths": 84,
    }


@strawberry.type
class Query:
    not_seen: "strawberry.Private[SensitiveData]"

    @strawberry.field
    def accessible_info(self) -> str:
        return self.not_seen.info


@dataclass
class SensitiveData:
    value: int
    info: str


def test_private_field_with_str_annotations():
    """Check compatibility of strawberry.Private with annotations as string."""
    schema = strawberry.Schema(query=Query)

    result = schema.execute_sync(
        "query { accessibleInfo }",
        root_value=Query(not_seen=SensitiveData(1, "foo")),
    )
    assert result.data == {"accessibleInfo": "foo"}

    # Check if querying `notSeen` raises error and no data is returned
    assert "notSeen" not in str(schema)
    failed_result = schema.execute_sync(
        "query { notSeen }", root_value=Query(not_seen=SensitiveData(1, "foo"))
    )
    assert failed_result.data is None


def test_private_field_defined_outside_module_scope():
    """Check compatibility of strawberry.Private when defined outside module scope."""
    global LocallyScopedSensitiveData

    @strawberry.type
    class LocallyScopedQuery:
        not_seen: "strawberry.Private[LocallyScopedSensitiveData]"

        @strawberry.field
        def accessible_info(self) -> str:
            return self.not_seen.info

    @dataclass
    class LocallyScopedSensitiveData:
        value: int
        info: str

    schema = strawberry.Schema(query=LocallyScopedQuery)

    assert "notSeen" not in str(schema)

    del LocallyScopedSensitiveData


def test_private_field_type_resolution_with_generic_type():
    """Check strawberry.Private when its argument is a implicit `Any` generic type.

    Refer to: https://github.com/strawberry-graphql/strawberry/issues/1938
    """
    T = TypeVar("T")

    class GenericPrivateType(Generic[T]):
        pass

    private_field = StrawberryField(
        type_annotation=StrawberryAnnotation(
            annotation="strawberry.Private[GenericPrivateType]",
            namespace={**globals(), **locals()},
        ),
    )
    assert private_field.type == strawberry.Private[GenericPrivateType]



from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

import strawberry

if TYPE_CHECKING:
    from starlette.testclient import TestClient


@pytest.fixture
def test_client() -> TestClient:
    from starlette.testclient import TestClient

    from strawberry.asgi import GraphQL

    @strawberry.type
    class Query:
        @strawberry.field
        async def hello(self, name: str | None = None) -> str:
            return f"Hello {name or 'world'}"

    async_schema = strawberry.Schema(Query)
    app = GraphQL[None, None](async_schema)
    return TestClient(app)


def test_simple_query(test_client: TestClient):
    response = test_client.post("/", json={"query": "{ hello }"})

    assert response.json() == {"data": {"hello": "Hello world"}}



from inline_snapshot import snapshot

from .utils.marks import requires_mypy, requires_pyright, requires_ty, skip_on_windows
from .utils.typecheck import Result, typecheck

pytestmark = [skip_on_windows, requires_pyright, requires_mypy, requires_ty]


CODE = """
import strawberry


@strawberry.type
class User:
    name: str


User("Patrick")

reveal_type(User.__init__)
"""


def test():
    results = typecheck(CODE)

    assert results.pyright == snapshot(
        [
            Result(
                type="error",
                message="Expected 0 positional arguments",
                line=10,
                column=6,
            ),
            Result(
                type="information",
                message='Type of "User.__init__" is "(self: User, *, name: str) -> None"',
                line=12,
                column=13,
            ),
        ]
    )
    assert results.mypy == snapshot(
        [
            Result(
                type="error",
                message='Too many positional arguments for "User"',
                line=10,
                column=1,
            ),
            Result(
                type="note",
                message='Revealed type is "def (self: mypy_test.User, *, name: str)"',
                line=12,
                column=13,
            ),
        ]
    )
    assert results.ty == snapshot(
        [
            Result(
                type="error",
                message="No argument provided for required parameter `name`",
                line=10,
                column=1,
            ),
            Result(
                type="error",
                message="Too many positional arguments: expected 0, got 1",
                line=10,
                column=6,
            ),
            Result(
                type="information",
                message="Revealed type: `(self: User, *, name: str) -> None`",
                line=12,
                column=13,
            ),
        ]
    )



from enum import Enum

import strawberry
from strawberry.annotation import StrawberryAnnotation


def test_basic():
    @strawberry.enum
    class NumaNuma(Enum):
        MA = "ma"
        I = "i"  # noqa: E741
        A = "a"
        HI = "hi"

    annotation = StrawberryAnnotation(NumaNuma)
    resolved = annotation.resolve()

    # TODO: Remove reference to .enum_definition with StrawberryEnumDefinition
    assert resolved is NumaNuma.__strawberry_definition__



from enum import Enum, IntEnum
from typing import Annotated

import pytest

import strawberry
from strawberry.exceptions import ObjectIsNotAnEnumError
from strawberry.types.base import get_object_definition
from strawberry.types.enum import StrawberryEnumDefinition


def test_basic_enum():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"

    definition = IceCreamFlavour.__strawberry_definition__

    assert definition.name == "IceCreamFlavour"
    assert definition.description is None

    assert definition.values[0].name == "VANILLA"
    assert definition.values[0].value == "vanilla"

    assert definition.values[1].name == "STRAWBERRY"
    assert definition.values[1].value == "strawberry"

    assert definition.values[2].name == "CHOCOLATE"
    assert definition.values[2].value == "chocolate"


def test_can_pass_name_and_description():
    @strawberry.enum(name="Flavour", description="example")
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"

    definition = IceCreamFlavour.__strawberry_definition__

    assert definition.name == "Flavour"
    assert definition.description == "example"


def test_can_use_enum_as_arguments():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"

    @strawberry.type
    class Query:
        @strawberry.field
        def flavour_available(self, flavour: IceCreamFlavour) -> bool:
            return flavour == IceCreamFlavour.STRAWBERRY

    field = Query.__strawberry_definition__.fields[0]

    assert isinstance(field.arguments[0].type, StrawberryEnumDefinition)


@pytest.mark.raises_strawberry_exception(
    ObjectIsNotAnEnumError,
    match="strawberry.enum can only be used with subclasses of Enum. ",
)
def test_raises_error_when_using_enum_with_a_not_enum_class():
    @strawberry.enum
    class AClass:
        hello = "world"


def test_can_deprecate_enum_values():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = strawberry.enum_value("vanilla")
        STRAWBERRY = strawberry.enum_value(
            "strawberry", deprecation_reason="We ran out"
        )
        CHOCOLATE = "chocolate"

    definition = IceCreamFlavour.__strawberry_definition__

    assert definition.values[0].name == "VANILLA"
    assert definition.values[0].value == "vanilla"
    assert definition.values[0].deprecation_reason is None

    assert definition.values[1].name == "STRAWBERRY"
    assert definition.values[1].value == "strawberry"
    assert definition.values[1].deprecation_reason == "We ran out"

    assert definition.values[2].name == "CHOCOLATE"
    assert definition.values[2].value == "chocolate"
    assert definition.values[2].deprecation_reason is None


def test_can_describe_enum_values():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = strawberry.enum_value("vanilla")
        STRAWBERRY = strawberry.enum_value(
            "strawberry",
            description="Our favourite",
        )
        CHOCOLATE = "chocolate"

    definition = IceCreamFlavour.__strawberry_definition__

    assert definition.values[0].name == "VANILLA"
    assert definition.values[0].value == "vanilla"
    assert definition.values[0].description is None

    assert definition.values[1].name == "STRAWBERRY"
    assert definition.values[1].value == "strawberry"
    assert definition.values[1].description == "Our favourite"

    assert definition.values[2].name == "CHOCOLATE"
    assert definition.values[2].value == "chocolate"
    assert definition.values[2].description is None


def test_can_use_enum_values():
    @strawberry.enum
    class TestEnum(Enum):
        A = "A"
        B = strawberry.enum_value("B")
        C = strawberry.enum_value("Coconut", deprecation_reason="We ran out")

    assert TestEnum.B.value == "B"

    assert [x.value for x in TestEnum.__members__.values()] == [
        "A",
        "B",
        "Coconut",
    ]


def test_int_enums():
    @strawberry.enum
    class TestEnum(IntEnum):
        A = 1
        B = 2
        C = 3
        D = strawberry.enum_value(4, description="D")

    assert TestEnum.A.value == 1
    assert TestEnum.B.value == 2
    assert TestEnum.C.value == 3
    assert TestEnum.D.value == 4

    assert [x.value for x in TestEnum.__members__.values()] == [1, 2, 3, 4]


def test_default_enum_implementation() -> None:
    class Foo(Enum):
        BAR = "bar"
        BAZ = "baz"

    @strawberry.type
    class Query:
        @strawberry.field
        def foo(self, foo: Foo) -> Foo:
            return foo

    schema = strawberry.Schema(Query)
    res = schema.execute_sync("{ foo(foo: BAR) }")
    assert not res.errors
    assert res.data
    assert res.data["foo"] == "BAR"


def test_default_enum_reuse() -> None:
    class Foo(Enum):
        BAR = "bar"
        BAZ = "baz"

    @strawberry.type
    class SomeType:
        foo: Foo
        bar: Foo

    definition = get_object_definition(SomeType, strict=True)
    assert definition.fields[1].type is definition.fields[1].type


def test_default_enum_with_enum_name_and_value() -> None:
    class Foo(Enum):
        FOO = "foo"
        BAR = strawberry.enum_value("bar")
        BAZ = strawberry.enum_value("baz", name="Baz")

    @strawberry.type
    class Query:
        @strawberry.field
        def foo(self, enums: list[Foo]) -> list[Foo]:
            assert enums == [Foo.FOO, Foo.BAR, Foo.BAZ]
            return enums

    schema = strawberry.Schema(Query)
    res = schema.execute_sync("{ foo(enums: [FOO, BAR, Baz]) }")
    assert not res.errors
    assert res.data
    assert res.data["foo"] == ["FOO", "BAR", "Baz"]


def test_use_enum_values() -> None:
    @strawberry.enum(graphql_name_from="value")
    class Foo(Enum):
        FOO = "foo"
        BAR = strawberry.enum_value("bar")
        BAZ = strawberry.enum_value("baz", name="Baz")

    @strawberry.type
    class Query:
        @strawberry.field
        def foo(self, enums: list[Foo]) -> list[Foo]:
            assert enums == [Foo.FOO, Foo.BAR, Foo.BAZ]
            return enums

    schema = strawberry.Schema(Query)
    res = schema.execute_sync("{ foo(enums: [foo, bar, Baz]) }")
    assert not res.errors
    assert res.data
    assert res.data["foo"] == ["foo", "bar", "Baz"]


def test_invalid_enum_value() -> None:
    @strawberry.enum(graphql_name_from="value")
    class Foo(Enum):
        FOO = 5  # Should be an string

    @strawberry.type
    class Query:
        foo: Foo

    with pytest.raises(TypeError, match=r"Expected name to be a string."):
        strawberry.Schema(Query)


def test_annotated_enum_with_description():
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"

    MyIceCreamFlavour = Annotated[
        IceCreamFlavour, strawberry.enum(description="Ice cream flavours")
    ]

    @strawberry.type
    class Query:
        @strawberry.field
        def flavour(self) -> MyIceCreamFlavour:
            return IceCreamFlavour.VANILLA

    schema = strawberry.Schema(Query)
    schema_str = str(schema)
    assert '"""Ice cream flavours"""' in schema_str
    assert "enum IceCreamFlavour" in schema_str


def test_annotated_enum_with_name():
    class Flavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"

    MyIceCreamFlavour = Annotated[Flavour, strawberry.enum(name="IceCreamFlavour")]

    @strawberry.type
    class Query:
        @strawberry.field
        def flavour(self) -> MyIceCreamFlavour:
            return Flavour.VANILLA

    schema = strawberry.Schema(Query)
    assert "IceCreamFlavour" in str(schema)


def test_annotated_enum_on_already_decorated():
    @strawberry.enum
    class Flavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"

    MyFlavour = Annotated[Flavour, strawberry.enum(description="Flavours")]

    @strawberry.type
    class Query:
        @strawberry.field
        def flavour(self) -> MyFlavour:
            return Flavour.VANILLA

    schema = strawberry.Schema(Query)
    assert "enum Flavour" in str(schema)


def test_annotated_enum_as_resolver_parameter():
    class Color(Enum):
        RED = "red"
        GREEN = "green"
        BLUE = "blue"

    GqlColor = Annotated[Color, strawberry.enum()]

    @strawberry.type
    class Query:
        @strawberry.field
        def color_name(self, color: GqlColor) -> str:
            return color.value

    schema = strawberry.Schema(query=Query)
    result = schema.execute_sync("{ colorName(color: RED) }")
    assert result.errors is None
    assert result.data == {"colorName": "red"}



from pydantic import BaseModel

import strawberry
from strawberry.federation.schema_directives import Key


def test_fetch_entities_pydantic():
    class ProductInDb(BaseModel):
        upc: str
        name: str

    # @strawberry.federation.type(keys=["upc"])
    @strawberry.experimental.pydantic.type(
        model=ProductInDb, directives=[Key(fields="upc", resolvable=True)]
    )
    class Product:
        upc: str
        name: str

        @classmethod
        def resolve_reference(cls, upc) -> "Product":
            return Product(upc=upc, name="")

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def top_products(self, first: int) -> list[Product]:  # pragma: no cover
            return []

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    upc
                }
            }
        }
    """

    result = schema.execute_sync(
        query,
        variable_values={
            "representations": [{"__typename": "Product", "upc": "B00005N5PF"}]
        },
    )

    assert not result.errors

    assert result.data == {"_entities": [{"upc": "B00005N5PF"}]}



# type: ignore

import textwrap

import strawberry
from strawberry.schema.config import StrawberryConfig


def test_field_provides_are_printed_correctly_camel_case_on():
    global Review

    @strawberry.federation.type
    class User:
        username: str

    @strawberry.federation.type(keys=["upc"], extend=True)
    class Product:
        upc: str = strawberry.federation.field(external=True)
        the_name: str = strawberry.federation.field(external=True)
        reviews: list["Review"]

    @strawberry.federation.type
    class Review:
        body: str
        author: User
        product: Product = strawberry.federation.field(provides=["name"])

    @strawberry.federation.type
    class Query:
        @strawberry.field
        def top_products(self, first: int) -> list[Product]:  # pragma: no cover
            return []

    schema = strawberry.federation.Schema(
        query=Query,
        config=StrawberryConfig(auto_camel_case=True),
    )

    expected = """
        schema @link(url: "https://specs.apollo.dev/federation/v2.11", import: ["@external", "@key", "@provides"]) {
          query: Query
        }

        extend type Product @key(fields: "upc") {
          upc: String! @external
          theName: String! @external
          reviews: [Review!]!
        }

        type Query {
          _entities(representations: [_Any!]!): [_Entity]!
          _service: _Service!
          topProducts(first: Int!): [Product!]!
        }

        type Review {
          body: String!
          author: User!
          product: Product! @provides(fields: "name")
        }

        type User {
          username: String!
        }

        scalar _Any

        union _Entity = Product

        type _Service {
          sdl: String!
        }
    """

    assert schema.as_str() == textwrap.dedent(expected).strip()

    del Review


def test_field_provides_are_printed_correctly_camel_case_off():
    global Review

    @strawberry.federation.type
    class User:
        username: str

    @strawberry.federation.type(keys=["upc"], extend=True)
    class Product:
        upc: str = strawberry.federation.field(external=True)
        the_name: str = strawberry.federation.field(external=True)
        reviews: list["Review"]

    @strawberry.federation.type
    class Review:
        body: str
        author: User
        product: Product = strawberry.federation.field(provides=["name"])

    @strawberry.federation.type
    class Query:
        @strawberry.field
        def top_products(self, first: int) -> list[Product]:  # pragma: no cover
            return []

    schema = strawberry.federation.Schema(
        query=Query,
        config=StrawberryConfig(auto_camel_case=False),
    )

    expected = """
        schema @link(url: "https://specs.apollo.dev/federation/v2.11", import: ["@external", "@key", "@provides"]) {
          query: Query
        }

        extend type Product @key(fields: "upc") {
          upc: String! @external
          the_name: String! @external
          reviews: [Review!]!
        }

        type Query {
          _entities(representations: [_Any!]!): [_Entity]!
          _service: _Service!
          top_products(first: Int!): [Product!]!
        }

        type Review {
          body: String!
          author: User!
          product: Product! @provides(fields: "name")
        }

        type User {
          username: String!
        }

        scalar _Any

        union _Entity = Product

        type _Service {
          sdl: String!
        }
    """

    assert schema.as_str() == textwrap.dedent(expected).strip()

    del Review



from textwrap import dedent
from typing import Annotated

import pytest

import strawberry


@pytest.fixture
def maybe_schema() -> strawberry.Schema:
    @strawberry.type
    class User:
        name: str
        phone: str | None

    user = User(name="Patrick", phone=None)

    @strawberry.type
    class Query:
        @strawberry.field
        def user(self) -> User:
            return user

    @strawberry.input
    class UpdateUserInput:
        phone: strawberry.Maybe[str | None]

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def update_user(self, input: UpdateUserInput) -> User:
            if input.phone:
                user.phone = input.phone.value
            return user

    return strawberry.Schema(query=Query, mutation=Mutation)


user_query = """
{
    user {
        phone
    }
}
"""


def set_phone(schema: strawberry.Schema, phone: str | None) -> dict:
    query = """
    mutation ($phone: String) {
        updateUser(input: { phone: $phone }) {
            phone
        }
    }
    """

    result = schema.execute_sync(query, variable_values={"phone": phone})
    assert not result.errors
    assert result.data
    return result.data["updateUser"]


def get_user(schema: strawberry.Schema) -> dict:
    result = schema.execute_sync(user_query)
    assert not result.errors
    assert result.data
    return result.data["user"]


def test_maybe(maybe_schema: strawberry.Schema) -> None:
    assert get_user(maybe_schema)["phone"] is None
    res = set_phone(maybe_schema, "123")
    assert res["phone"] == "123"


def test_maybe_some_to_none(maybe_schema: strawberry.Schema) -> None:
    assert get_user(maybe_schema)["phone"] is None
    set_phone(maybe_schema, "123")
    res = set_phone(maybe_schema, None)
    assert res["phone"] is None


def test_maybe_absent_value(maybe_schema: strawberry.Schema) -> None:
    set_phone(maybe_schema, "123")

    query = """
    mutation {
        updateUser(input: {}) {
            phone
        }
    }
    """
    result = maybe_schema.execute_sync(query)
    assert not result.errors
    assert result.data
    assert result.data["updateUser"]["phone"] == "123"
    # now check the reverse case.

    set_phone(maybe_schema, None)
    result = maybe_schema.execute_sync(query)
    assert not result.errors
    assert result.data
    assert result.data["updateUser"]["phone"] is None


def test_optional_argument_maybe() -> None:
    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self, name: strawberry.Maybe[str | None] = None) -> str:
            if name:
                return "None" if name.value is None else name.value

            return "UNSET"

    schema = strawberry.Schema(query=Query)

    assert str(schema) == dedent(
        """\
        type Query {
          hello(name: String): String!
        }"""
    )

    result = schema.execute_sync(
        """
        query {
            hello
        }
    """
    )
    assert not result.errors
    assert result.data == {"hello": "UNSET"}
    result = schema.execute_sync(
        """
        query {
            hello(name: "bar")
        }
    """
    )
    assert not result.errors
    assert result.data == {"hello": "bar"}
    result = schema.execute_sync(
        """
        query {
            hello(name: null)
        }
    """
    )
    assert not result.errors
    assert result.data == {"hello": "None"}


def test_maybe_list():
    @strawberry.input
    class InputData:
        items: strawberry.Maybe[list[str] | None]

    @strawberry.type
    class Query:
        @strawberry.field
        def test(self, data: InputData) -> str:
            return "I am a test, and I received: " + str(data.items)

    schema = strawberry.Schema(Query)

    assert str(schema) == dedent(
        """\
        input InputData {
          items: [String!]
        }

        type Query {
          test(data: InputData!): String!
        }"""
    )


def test_maybe_str_rejects_explicit_nulls():
    """Test that Maybe[str] correctly rejects null values at Python validation level.

    BEHAVIOR:
    - Maybe[str] generates 'String' (optional) in GraphQL schema
    - Rejects null values at Python validation level, not GraphQL level
    - This allows the GraphQL parser to accept null, but Python validation rejects it
    """

    @strawberry.input
    class UpdateInput:
        name: strawberry.Maybe[str]  # Should be optional in schema but reject null

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def update(self, input: UpdateInput) -> str:
            if input.name is not None:
                return f"Updated to: {input.name.value}"
            return "No change"

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    # Generates optional field
    schema_str = str(schema)
    assert "name: String" in schema_str
    assert "name: String!" not in schema_str

    # Test with explicit null fails at Python validation level
    query = """
    mutation {
        update(input: { name: null })
    }
    """

    result = schema.execute_sync(query)
    assert result.errors
    assert len(result.errors) == 1
    # The error should mention Python-level validation, not GraphQL schema validation
    error_message = str(result.errors[0])
    assert "Expected value of type" in error_message
    assert "found null" in error_message


def test_maybe_str_accepts_valid_values():
    """Test that Maybe[str] accepts valid string values."""

    @strawberry.input
    class UpdateInput:
        name: strawberry.Maybe[str]

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def update(self, input: UpdateInput) -> str:
            if input.name is not None:
                return f"Updated to: {input.name.value}"
            return "No change"

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    # Test with valid string value should work
    query = """
    mutation {
        update(input: { name: "John" })
    }
    """

    result = schema.execute_sync(query)
    assert not result.errors
    assert result.data == {"update": "Updated to: John"}


def test_maybe_str_handles_absent_fields():
    """Test that Maybe[str] properly handles absent fields.

    BEHAVIOR:
    - Absent fields are allowed and result in None value
    """

    @strawberry.input
    class UpdateInput:
        name: strawberry.Maybe[str]

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def update(self, input: UpdateInput) -> str:
            if input.name is not None:
                return f"Updated to: {input.name.value}"
            return "No change"

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    # Test with absent field should work and return "No change"
    query = """
    mutation {
        update(input: {})
    }
    """

    result = schema.execute_sync(query)
    assert not result.errors
    assert result.data == {"update": "No change"}


def test_maybe_str_error_messages():
    """Test that Maybe[str] provides error messages when null is rejected."""

    @strawberry.input
    class UpdateInput:
        name: strawberry.Maybe[str]  # Rejects null at Python validation level
        phone: strawberry.Maybe[str | None]  # Can accept null

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def update(self, input: UpdateInput) -> str:
            return "Updated"

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    # Test that null to name field produces a validation error
    query = """
    mutation {
        update(input: { name: null, phone: null })
    }
    """

    result = schema.execute_sync(query)
    assert result.errors
    assert len(result.errors) == 1

    # The error should be related to the name field, not phone field
    error_message = str(result.errors[0])
    # Error message from Python validation
    assert "Expected value of type" in error_message
    assert "found null" in error_message


def test_mixed_maybe_field_behavior():
    """Test schema with both Maybe[T] and Maybe[T | None] behave differently."""

    @strawberry.input
    class UpdateUserInput:
        # Should accept value or absent, reject null
        username: strawberry.Maybe[str]
        # Can accept null, value, or absent
        bio: strawberry.Maybe[str | None]
        # Can accept null, value, or absent
        website: strawberry.Maybe[str | None]

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def update_user(self, input: UpdateUserInput) -> str:
            result = []

            if input.username is not None:
                result.append(f"username={input.username.value}")
            else:
                result.append("username=unchanged")

            if input.bio is not None:
                bio_value = (
                    input.bio.value if input.bio.value is not None else "cleared"
                )
                result.append(f"bio={bio_value}")
            else:
                result.append("bio=unchanged")

            if input.website is not None:
                website_value = (
                    input.website.value
                    if input.website.value is not None
                    else "cleared"
                )
                result.append(f"website={website_value}")
            else:
                result.append("website=unchanged")

            return ", ".join(result)

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    # Test 1: Valid values for all fields
    query1 = """
    mutation {
        updateUser(input: {
            username: "john",
            bio: "Developer",
            website: "example.com"
        })
    }
    """
    result1 = schema.execute_sync(query1)
    assert not result1.errors
    assert result1.data == {
        "updateUser": "username=john, bio=Developer, website=example.com"
    }

    # Test 2: Null for bio and website should work, but not for username
    query2 = """
    mutation {
        updateUser(input: {
            username: null,
            bio: null,
            website: null
        })
    }
    """
    result2 = schema.execute_sync(query2)
    assert result2.errors  # Should fail due to username: null

    # Test 3: Valid bio/website nulls without username
    query3 = """
    mutation {
        updateUser(input: {
            bio: null,
            website: null
        })
    }
    """
    result3 = schema.execute_sync(query3)
    assert not result3.errors
    assert result3.data == {
        "updateUser": "username=unchanged, bio=cleared, website=cleared"
    }

    # Test 4: Absent fields should work
    query4 = """
    mutation {
        updateUser(input: {})
    }
    """
    result4 = schema.execute_sync(query4)
    assert not result4.errors
    assert result4.data == {
        "updateUser": "username=unchanged, bio=unchanged, website=unchanged"
    }


def test_maybe_nested_types():
    """Test Maybe with nested types like lists."""

    @strawberry.input
    class UpdateItemsInput:
        # Cannot accept null list - only valid list or absent
        tags: strawberry.Maybe[list[str]]
        # Can accept null, valid list, or absent
        categories: strawberry.Maybe[list[str] | None]

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def update_items(self, input: UpdateItemsInput) -> str:
            result = []

            if input.tags is not None:
                result.append(f"tags={input.tags.value}")
            else:
                result.append("tags=unchanged")

            if input.categories is not None:
                cat_value = (
                    input.categories.value
                    if input.categories.value is not None
                    else "cleared"
                )
                result.append(f"categories={cat_value}")
            else:
                result.append("categories=unchanged")

            return ", ".join(result)

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    # Test 1: Valid lists for both fields
    query1 = """
    mutation {
        updateItems(input: {
            tags: ["python", "graphql"],
            categories: ["tech", "web"]
        })
    }
    """
    result1 = schema.execute_sync(query1)
    assert not result1.errors
    assert result1.data == {
        "updateItems": "tags=['python', 'graphql'], categories=['tech', 'web']"
    }

    # Test 2: Null categories should work, but null tags should fail
    query2 = """
    mutation {
        updateItems(input: {
            tags: null,
            categories: null
        })
    }
    """
    result2 = schema.execute_sync(query2)
    assert result2.errors  # Should fail due to tags: null

    # Test 3: Valid categories null without tags
    query3 = """
    mutation {
        updateItems(input: {
            categories: null
        })
    }
    """
    result3 = schema.execute_sync(query3)
    assert not result3.errors
    assert result3.data == {"updateItems": "tags=unchanged, categories=cleared"}

    # Test 4: Empty lists should work for both
    query4 = """
    mutation {
        updateItems(input: {
            tags: [],
            categories: []
        })
    }
    """
    result4 = schema.execute_sync(query4)
    assert not result4.errors
    assert result4.data == {"updateItems": "tags=[], categories=[]"}


def test_maybe_resolver_arguments():
    """Test Maybe fields as resolver arguments."""

    @strawberry.type
    class Query:
        @strawberry.field
        def search(
            self,
            # Cannot accept null - only value or absent
            query: strawberry.Maybe[str] = None,
            # Can accept null, value, or absent
            filter_by: strawberry.Maybe[str | None] = None,
        ) -> str:
            result = []

            if query is not None:
                result.append(f"query={query.value}")
            else:
                result.append("query=unset")

            if filter_by is not None:
                filter_value = (
                    filter_by.value if filter_by.value is not None else "cleared"
                )
                result.append(f"filter={filter_value}")
            else:
                result.append("filter=unset")

            return ", ".join(result)

    schema = strawberry.Schema(query=Query)

    # Test 1: Valid values for both arguments
    query1 = """
    query {
        search(query: "python", filterBy: "category")
    }
    """
    result1 = schema.execute_sync(query1)
    assert not result1.errors
    assert result1.data == {"search": "query=python, filter=category"}

    # Test 2: Null filter should work, but null query should fail
    query2 = """
    query {
        search(query: null, filterBy: null)
    }
    """
    result2 = schema.execute_sync(query2)
    assert result2.errors  # Should fail due to query: null

    # Test 3: Valid filter null without query
    query3 = """
    query {
        search(filterBy: null)
    }
    """
    result3 = schema.execute_sync(query3)
    assert not result3.errors
    assert result3.data == {"search": "query=unset, filter=cleared"}

    # Test 4: No arguments
    query4 = """
    query {
        search
    }
    """
    result4 = schema.execute_sync(query4)
    assert not result4.errors
    assert result4.data == {"search": "query=unset, filter=unset"}


def test_maybe_graphql_schema_consistency():
    """Test GraphQL schema generation for Maybe types.

    BEHAVIOR:
    - Maybe[str] generates String (optional) - allows absent but rejects null
    - Maybe[str | None] generates String (optional) - allows absent and null
    - Both generate the same GraphQL schema but have different validation
    """

    # Schema with Maybe[str]
    @strawberry.input
    class Input1:
        field: strawberry.Maybe[str]

    @strawberry.type
    class Query1:
        @strawberry.field
        def test(self, input: Input1) -> str:
            return "test"

    schema1 = strawberry.Schema(query=Query1)

    # Schema with Maybe[str | None]
    @strawberry.input
    class Input2:
        field: strawberry.Maybe[str | None]

    @strawberry.type
    class Query2:
        @strawberry.field
        def test(self, input: Input2) -> str:
            return "test"

    schema2 = strawberry.Schema(query=Query2)

    # Document new behavior
    schema1_str = str(schema1)
    schema2_str = str(schema2)

    # Both Maybe[str] and Maybe[str | None] generate String (optional)
    assert "field: String" in schema1_str
    assert "field: String!" not in schema1_str
    assert "field: String" in schema2_str
    assert "field: String!" not in schema2_str


def test_maybe_complex_types():
    """Test Maybe with complex custom types."""

    @strawberry.input
    class AddressInput:
        street: str
        city: str
        zip_code: str | None = None

    @strawberry.input
    class UpdateProfileInput:
        # Cannot accept null address - only valid address or absent
        address: strawberry.Maybe[AddressInput]
        # Can accept null, valid address, or absent
        billing_address: strawberry.Maybe[AddressInput | None]

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def update_profile(self, input: UpdateProfileInput) -> str:
            result = []

            if input.address is not None:
                addr = input.address.value
                result.append(f"address={addr.street}, {addr.city}")
            else:
                result.append("address=unchanged")

            if input.billing_address is not None:
                if input.billing_address.value is not None:
                    addr = input.billing_address.value
                    result.append(f"billing={addr.street}, {addr.city}")
                else:
                    result.append("billing=cleared")
            else:
                result.append("billing=unchanged")

            return ", ".join(result)

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    # Test 1: Valid addresses for both fields
    query1 = """
    mutation {
        updateProfile(input: {
            address: { street: "123 Main", city: "NYC" },
            billingAddress: { street: "456 Oak", city: "LA" }
        })
    }
    """
    result1 = schema.execute_sync(query1)
    assert not result1.errors
    assert result1.data == {
        "updateProfile": "address=123 Main, NYC, billing=456 Oak, LA"
    }

    # Test 2: Null billing should work, but null address should fail
    query2 = """
    mutation {
        updateProfile(input: {
            address: null,
            billingAddress: null
        })
    }
    """
    result2 = schema.execute_sync(query2)
    assert result2.errors  # Should fail due to address: null

    # Test 3: Valid billing null without address
    query3 = """
    mutation {
        updateProfile(input: {
            billingAddress: null
        })
    }
    """
    result3 = schema.execute_sync(query3)
    assert not result3.errors
    assert result3.data == {"updateProfile": "address=unchanged, billing=cleared"}

    # Test 4: Absent fields
    query4 = """
    mutation {
        updateProfile(input: {})
    }
    """
    result4 = schema.execute_sync(query4)
    assert not result4.errors
    assert result4.data == {"updateProfile": "address=unchanged, billing=unchanged"}


def test_maybe_union_with_none_works():
    """Test that Maybe[T | None] works correctly (this should pass)."""

    @strawberry.input
    class TestInput:
        # This should work correctly - can accept value, null, or absent
        field: strawberry.Maybe[str | None]

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def test(self, input: TestInput) -> str:
            if input.field is not None:
                if input.field.value is not None:
                    return f"value={input.field.value}"
                return "null"
            return "absent"

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    # Schema should show optional field
    schema_str = str(schema)
    assert "field: String" in schema_str
    assert "field: String!" not in schema_str

    # Test valid value
    result1 = schema.execute_sync('mutation { test(input: { field: "hello" }) }')
    assert not result1.errors
    assert result1.data == {"test": "value=hello"}

    # Test null value
    result2 = schema.execute_sync("mutation { test(input: { field: null }) }")
    assert not result2.errors
    assert result2.data == {"test": "null"}

    # Test absent field
    result3 = schema.execute_sync("mutation { test(input: {}) }")
    assert not result3.errors
    assert result3.data == {"test": "absent"}


def test_maybe_behavior_documented():
    """Document the behavior of Maybe[str] vs Maybe[str | None]."""

    @strawberry.input
    class CompareInput:
        # Generates String (optional) but rejects null at Python level
        required_field: strawberry.Maybe[str]
        # Generates String (optional) and accepts null
        optional_field: strawberry.Maybe[str | None]

    @strawberry.type
    class Query:
        @strawberry.field
        def compare(self, input: CompareInput) -> str:
            return "test"

    schema = strawberry.Schema(query=Query)
    schema_str = str(schema)

    # Document behavior - both generate optional fields
    assert "requiredField: String" in schema_str
    assert "requiredField: String!" not in schema_str
    assert "optionalField: String" in schema_str
    assert "optionalField: String!" not in schema_str


def test_maybe_schema_generation():
    """Test schema behavior - both Maybe[T] and Maybe[T | None] generate optional."""

    @strawberry.input
    class Input1:
        field: strawberry.Maybe[str]

    @strawberry.input
    class Input2:
        field: strawberry.Maybe[str | None]

    @strawberry.type
    class Query:
        @strawberry.field
        def test1(self, input: Input1) -> str:
            return "test"

        @strawberry.field
        def test2(self, input: Input2) -> str:
            return "test"

    schema = strawberry.Schema(query=Query)
    schema_str = str(schema)

    # Both Maybe[str] and Maybe[str | None] generate String (optional)
    assert "field: String" in schema_str  # Both Input1 and Input2 have optional fields
    assert "field: String!" not in schema_str  # No required fields


def test_maybe_validation():
    """Test that Maybe[T] rejects null at Python validation time, not GraphQL schema time."""

    @strawberry.input
    class TestInput:
        field: strawberry.Maybe[
            str
        ]  # Should be optional in schema but reject null in validation

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def test(self, input: TestInput) -> str:
            return "test"

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    # Schema should show optional field
    schema_str = str(schema)
    assert "field: String" in schema_str
    assert "field: String!" not in schema_str

    # Null should be rejected during Python validation, not GraphQL parsing
    result = schema.execute_sync("mutation { test(input: { field: null }) }")
    assert result.errors
    # Error should be a validation error, not a GraphQL parsing error
    error_message = str(result.errors[0])
    assert "Expected value of type" in error_message
    assert "found null" in error_message


def test_maybe_comprehensive_behavior_comparison():
    """Comprehensive test comparing Maybe[T] vs Maybe[T | None] behavior."""

    @strawberry.input
    class ComprehensiveInput:
        # String (optional) - can be value or absent, but rejects null at Python level
        strict_field: strawberry.Maybe[str]
        # String (optional) - can be null, value, or absent
        flexible_field: strawberry.Maybe[str | None]

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def test_comprehensive(self, input: ComprehensiveInput) -> str:
            result = []

            # This logic works for both current and intended behavior
            if input.strict_field is not None:
                result.append(f"strict={input.strict_field.value}")
            else:
                result.append("strict=absent")

            if input.flexible_field is not None:
                if input.flexible_field.value is not None:
                    result.append(f"flexible={input.flexible_field.value}")
                else:
                    result.append("flexible=null")
            else:
                result.append("flexible=absent")

            return ", ".join(result)

    schema = strawberry.Schema(query=Query, mutation=Mutation)
    schema_str = str(schema)

    # Document schema generation - both are optional
    assert "strictField: String" in schema_str
    assert "strictField: String!" not in schema_str
    assert "flexibleField: String" in schema_str
    assert "flexibleField: String!" not in schema_str

    # Test 1: Valid values work for both
    result1 = schema.execute_sync("""
        mutation {
            testComprehensive(input: {
                strictField: "hello",
                flexibleField: "world"
            })
        }
    """)
    assert not result1.errors
    assert result1.data == {"testComprehensive": "strict=hello, flexible=world"}

    # Test 2: Only flexible field can be null
    result2 = schema.execute_sync("""
        mutation {
            testComprehensive(input: {
                strictField: "hello",
                flexibleField: null
            })
        }
    """)
    assert not result2.errors
    assert result2.data == {"testComprehensive": "strict=hello, flexible=null"}

    # Test 3: Strict field null causes Python validation error
    result3 = schema.execute_sync("""
        mutation {
            testComprehensive(input: {
                strictField: null,
                flexibleField: "world"
            })
        }
    """)
    assert result3.errors  # Python validation error - cannot pass null to Maybe[str]

    # Test 4: Both fields can be omitted now
    result4 = schema.execute_sync("""
        mutation {
            testComprehensive(input: {
                strictField: "hello"
            })
        }
    """)
    assert not result4.errors
    assert result4.data == {"testComprehensive": "strict=hello, flexible=absent"}

    # Test 5: Strict field can now be omitted (both fields optional in schema)
    result5 = schema.execute_sync("""
        mutation {
            testComprehensive(input: {
                flexibleField: "world"
            })
        }
    """)
    assert not result5.errors  # No error - both fields are optional in schema
    assert result5.data == {"testComprehensive": "strict=absent, flexible=world"}


def test_maybe_with_explicit_field_description():
    """Handle case where strawberry.field annotation is used on a field with Maybe[T] type."""

    @strawberry.input
    class InputData:
        name: strawberry.Maybe[str | None] = strawberry.field(
            description="This strawberry.field annotation was breaking in default injection"
        )

    @strawberry.type
    class Query:
        @strawberry.field
        def test(self, data: InputData) -> str:
            if data.name is None:
                return "I am a test, and I received: None"
            return "I am a test, and I received: " + str(data.name.value)

    schema = strawberry.Schema(Query)

    assert str(schema) == dedent(
        '''\
        input InputData {
          """This strawberry.field annotation was breaking in default injection"""
          name: String
        }

        type Query {
          test(data: InputData!): String!
        }'''
    )

    query1 = """
    query {
        test(data: { name: null })
    }
    """
    result1 = schema.execute_sync(query1)
    assert not result1.errors

    query2 = """
    query {
        test(data: { name: "hello" })
    }
    """
    result2 = schema.execute_sync(query2)
    assert not result2.errors

    query3 = """
    query {
        test(data: {})
    }
    """
    result3 = schema.execute_sync(query3)
    assert not result3.errors


def test_maybe_wrapped_with_annotated_typing():
    """Handle case where Maybe is wrapped with Annotated typing."""

    @strawberry.input
    class InputData:
        name: Annotated[strawberry.Maybe[str | None], "some meta"]

    @strawberry.type
    class Query:
        @strawberry.field
        def test(self, data: InputData) -> str:
            if data.name is None:
                return "I am a test, and I received: None"
            return "I am a test, and I received: " + str(data.name.value)

    schema = strawberry.Schema(Query)

    assert str(schema) == dedent(
        """\
        input InputData {
          name: String
        }

        type Query {
          test(data: InputData!): String!
        }"""
    )
    query1 = """
    query {
        test(data: { name: null })
    }
    """
    result1 = schema.execute_sync(query1)
    assert not result1.errors

    query2 = """
    query {
        test(data: { name: "hello" })
    }
    """
    result2 = schema.execute_sync(query2)
    assert not result2.errors

    query3 = """
    query {
        test(data: {})
    }
    """
    result3 = schema.execute_sync(query3)
    assert not result3.errors


def test_maybe_with_annotated_and_explicit_definition():
    """Handle case where Maybe is wrapped with Annotated typing."""

    @strawberry.input
    class InputData:
        name: Annotated[strawberry.Maybe[str | None], "some meta"] = strawberry.field(
            description="This strawberry.field annotation was breaking in default injection"
        )

    @strawberry.type
    class Query:
        @strawberry.field
        def test(self, data: InputData) -> str:
            if data.name is None:
                return "I am a test, and I received: None"
            return "I am a test, and I received: " + str(data.name.value)

    schema = strawberry.Schema(Query)

    assert str(schema) == dedent(
        '''\
        input InputData {
          """This strawberry.field annotation was breaking in default injection"""
          name: String
        }

        type Query {
          test(data: InputData!): String!
        }'''
    )
    query1 = """
    query {
        test(data: { name: null })
    }
    """
    result1 = schema.execute_sync(query1)
    assert not result1.errors

    query2 = """
    query {
        test(data: { name: "hello" })
    }
    """
    result2 = schema.execute_sync(query2)
    assert not result2.errors

    query3 = """
    query {
        test(data: {})
    }
    """
    result3 = schema.execute_sync(query3)
    assert not result3.errors


def test_maybe_str_rejects_null_literals():
    """Test that Maybe[str] rejects literal null values in GraphQL queries."""

    @strawberry.input
    class UpdateInput:
        name: strawberry.Maybe[str]

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def update(self, input: UpdateInput) -> str:
            if input.name is not None:
                return f"Updated to: {input.name.value}"
            return "No change"

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    query = """
    mutation {
        update(input: { name: null })
    }
    """

    result = schema.execute_sync(query)
    assert result.errors
    assert len(result.errors) == 1
    error_message = str(result.errors[0])
    assert "Expected value of type 'str', found null" in error_message
    assert "name" in error_message
    assert "cannot be explicitly set to null" in error_message

    query_valid = """
    mutation {
        update(input: { name: "John" })
    }
    """

    result_valid = schema.execute_sync(query_valid)
    assert not result_valid.errors
    assert result_valid.data == {"update": "Updated to: John"}

    query_omitted = """
    mutation {
        update(input: {})
    }
    """

    result_omitted = schema.execute_sync(query_omitted)
    assert not result_omitted.errors
    assert result_omitted.data == {"update": "No change"}


def test_maybe_str_rejects_null_via_variables():
    """Test that Maybe[str] rejects null values passed through variables.

    This ensures JSON-based integrations (Flask, FastAPI, etc.) that pass
    inputs as variables properly validate null values.
    """

    @strawberry.input
    class UpdateInput:
        name: strawberry.Maybe[str]

    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def update(self, input: UpdateInput) -> str:
            if input.name is not None:
                return f"Updated to: {input.name.value}"
            return "No change"

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    query_with_vars = """
    mutation($input: UpdateInput!) {
        update(input: $input)
    }
    """

    variables = {"input": {"name": None}}

    result = schema.execute_sync(query_with_vars, variable_values=variables)
    assert result.errors
    assert len(result.errors) == 1
    error_message = str(result.errors[0])
    assert "Expected value of type 'str', found null" in error_message
    assert "cannot be explicitly set to null" in error_message

    variables_valid = {"input": {"name": "John"}}
    result_valid = schema.execute_sync(query_with_vars, variable_values=variables_valid)
    assert not result_valid.errors
    assert result_valid.data == {"update": "Updated to: John"}

    variables_omitted = {"input": {}}
    result_omitted = schema.execute_sync(
        query_with_vars, variable_values=variables_omitted
    )
    assert not result_omitted.errors
    assert result_omitted.data == {"update": "No change"}

    @strawberry.input
    class UpdateInputFlexible:
        name: strawberry.Maybe[str | None]

    @strawberry.type
    class Mutation2:
        @strawberry.mutation
        def update_flexible(self, input: UpdateInputFlexible) -> str:
            if input.name is not None:
                if input.name.value is None:
                    return "Cleared"
                return f"Updated to: {input.name.value}"
            return "No change"

    schema2 = strawberry.Schema(query=Query, mutation=Mutation2)

    query_flexible = """
    mutation($input: UpdateInputFlexible!) {
        updateFlexible(input: $input)
    }
    """

    variables_null = {"input": {"name": None}}
    result_null = schema2.execute_sync(query_flexible, variable_values=variables_null)
    assert not result_null.errors
    assert result_null.data == {"updateFlexible": "Cleared"}



import pytest

from strawberry.schema.config import StrawberryConfig
from strawberry.types.info import Info


def test_config_post_init_auto_camel_case():
    config = StrawberryConfig(auto_camel_case=True)

    assert config.name_converter.auto_camel_case is True


def test_config_post_init_no_auto_camel_case():
    config = StrawberryConfig(auto_camel_case=False)

    assert config.name_converter.auto_camel_case is False


def test_config_post_init_info_class():
    class CustomInfo(Info):
        test: str = "foo"

    config = StrawberryConfig(info_class=CustomInfo)

    assert config.info_class is CustomInfo
    assert config.info_class.test == "foo"


def test_config_post_init_info_class_is_default():
    config = StrawberryConfig()

    assert config.info_class is Info


def test_config_post_init_info_class_is_not_subclass():
    with pytest.raises(TypeError) as exc_info:
        StrawberryConfig(info_class=object)

    assert str(exc_info.value) == "`info_class` must be a subclass of strawberry.Info"



from __future__ import annotations

import textwrap

import strawberry
from strawberry.schema_directives import OneOf


@strawberry.input(directives=[OneOf()])
class ExampleInputTagged:
    a: str | None
    b: int | None


@strawberry.type
class ExampleResult:
    a: str | None
    b: int | None


@strawberry.type
class Query:
    @strawberry.field
    def test(self, input: ExampleInputTagged) -> ExampleResult:  # pragma: no cover
        return input  # type: ignore


schema = strawberry.Schema(query=Query)


def test_prints_one_of_directive():
    expected_type = """
    directive @oneOf on INPUT_OBJECT

    input ExampleInputTagged @oneOf {
      a: String
      b: Int
    }

    type ExampleResult {
      a: String
      b: Int
    }

    type Query {
      test(input: ExampleInputTagged!): ExampleResult!
    }
    """

    assert str(schema) == textwrap.dedent(expected_type).strip()



import itertools
from enum import Enum
from typing import Optional, TypeVar

import pytest

import strawberry
from strawberry.annotation import StrawberryAnnotation
from strawberry.types.unset import UnsetType


class Bleh:
    pass


@strawberry.enum
class NumaNuma(Enum):
    MA = "ma"
    I = "i"  # noqa: E741
    A = "a"
    HI = "hi"


T = TypeVar("T")

types = [
    int,
    str,
    None,
    Optional[str],
    UnsetType,
    "int",
    T,
    Bleh,
    NumaNuma,
]


@pytest.mark.parametrize(
    ("type1", "type2"), itertools.combinations_with_replacement(types, 2)
)
def test_annotation_hash(type1: object | str, type2: object | str):
    annotation1 = StrawberryAnnotation(type1)
    annotation2 = StrawberryAnnotation(type2)
    assert (
        hash(annotation1) == hash(annotation2)
        if annotation1 == annotation2
        else hash(annotation1) != hash(annotation2)
    ), "Equal type must imply equal hash"


def test_eq_on_other_type():
    class Foo:  # noqa: PLW1641
        def __eq__(self, other):
            # Anything that is a strawberry annotation is equal to Foo
            return isinstance(other, StrawberryAnnotation)

    assert Foo() != object()
    assert object() != Foo()
    assert Foo() != 123 != Foo()
    assert Foo() != 123
    assert Foo() == StrawberryAnnotation(int)
    assert StrawberryAnnotation(int) == Foo()


def test_eq_on_non_annotation():
    assert StrawberryAnnotation(int) is not int
    assert StrawberryAnnotation(int) != 123


def test_set_anntation():
    annotation = StrawberryAnnotation(int)
    annotation.annotation = str

    assert annotation.annotation is str



import contextlib
import dataclasses
import enum
from collections.abc import AsyncGenerator
from typing import Any

import pytest

import strawberry
from strawberry.extensions import SchemaExtension


@dataclasses.dataclass
class SchemaHelper:
    query_type: type
    subscription_type: type
    query: str
    subscription: str


class ExampleExtension(SchemaExtension):
    def __init_subclass__(cls, **kwargs: Any):
        super().__init_subclass__(**kwargs)
        cls.called_hooks = []

    expected = [
        "on_operation Entered",
        "on_parse Entered",
        "on_parse Exited",
        "on_validate Entered",
        "on_validate Exited",
        "on_execute Entered",
        "resolve",
        "resolve",
        "on_execute Exited",
        "on_operation Exited",
        "get_results",
    ]
    called_hooks: list[str]

    @classmethod
    def assert_expected(cls) -> None:
        assert cls.called_hooks == cls.expected


@pytest.fixture
def default_query_types_and_query() -> SchemaHelper:
    @strawberry.type
    class Person:
        name: str = "Jess"

    @strawberry.type
    class Query:
        @strawberry.field
        def person(self) -> Person:
            return Person()

    @strawberry.type
    class Subscription:
        @strawberry.subscription
        async def count(self) -> AsyncGenerator[int, None]:
            for i in range(5):
                yield i

    subscription = "subscription TestSubscribe { count }"
    query = "query TestQuery { person { name } }"
    return SchemaHelper(
        query_type=Query,
        query=query,
        subscription_type=Subscription,
        subscription=subscription,
    )


class ExecType(enum.Enum):
    SYNC = enum.auto()
    ASYNC = enum.auto()

    def is_async(self) -> bool:
        return self == ExecType.ASYNC


@pytest.fixture(params=[ExecType.ASYNC, ExecType.SYNC])
def exec_type(request: pytest.FixtureRequest) -> ExecType:
    return request.param


@contextlib.contextmanager
def hook_wrap(list_: list[str], hook_name: str):
    list_.append(f"{hook_name} Entered")
    try:
        yield
    finally:
        list_.append(f"{hook_name} Exited")


@pytest.fixture
def async_extension() -> type[ExampleExtension]:
    class MyExtension(ExampleExtension):
        async def on_operation(self):
            with hook_wrap(self.called_hooks, SchemaExtension.on_operation.__name__):
                yield

        async def on_validate(self):
            with hook_wrap(self.called_hooks, SchemaExtension.on_validate.__name__):
                yield

        async def on_parse(self):
            with hook_wrap(self.called_hooks, SchemaExtension.on_parse.__name__):
                yield

        async def on_execute(self):
            with hook_wrap(self.called_hooks, SchemaExtension.on_execute.__name__):
                yield

        async def get_results(self):
            self.called_hooks.append("get_results")
            return {"example": "example"}

        async def resolve(self, _next, root, info, *args: str, **kwargs: Any):
            self.called_hooks.append("resolve")
            return _next(root, info, *args, **kwargs)

    return MyExtension



import asyncio
from collections.abc import AsyncIterator

import pytest
from graphql import ExecutionResult
from pytest_codspeed.plugin import BenchmarkFixture

from .api import schema


@pytest.mark.benchmark
def test_subscription(benchmark: BenchmarkFixture):
    s = """
    subscription {
        something
    }
    """

    async def _run():
        for _ in range(100):
            iterator = await schema.subscribe(s)

            value = await iterator.__anext__()  # type: ignore[union-attr]

            assert value.data is not None
            assert value.data["something"] == "Hello World!"

    benchmark(lambda: asyncio.run(_run()))


@pytest.mark.benchmark
@pytest.mark.parametrize("count", [1000, 20000])
def test_subscription_long_run(benchmark: BenchmarkFixture, count: int) -> None:
    s = """#graphql
    subscription LongRunning($count: Int!) {
        longRunning(count: $count)
    }
    """

    async def _run():
        i = 0
        aiterator: AsyncIterator[ExecutionResult] = await schema.subscribe(
            s, variable_values={"count": count}
        )  # type: ignore[assignment]
        async for res in aiterator:
            assert res.data is not None
            assert res.data["longRunning"] == i
            i += 1

    benchmark(lambda: asyncio.run(_run()))



import dataclasses
import textwrap
from operator import getitem

import strawberry
from strawberry.printer import print_schema
from strawberry.schema.config import StrawberryConfig
from strawberry.types.field import StrawberryField


def test_custom_field():
    class CustomField(StrawberryField):
        def get_result(self, root, info, args, kwargs):
            return getattr(root, self.python_name) * 2

    @strawberry.type
    class Query:
        a: str = CustomField(default="Example")  # type: ignore

    schema = strawberry.Schema(query=Query)

    query = "{ a }"

    result = schema.execute_sync(query, root_value=Query())

    assert not result.errors
    assert result.data == {"a": "ExampleExample"}


def test_default_resolver_gets_attribute():
    @strawberry.type
    class User:
        name: str

    @strawberry.type
    class Query:
        @strawberry.field
        def user(self) -> User:
            return User(name="Patrick")

    schema = strawberry.Schema(query=Query)

    query = "{ user { name } }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data
    assert result.data["user"]["name"] == "Patrick"


def test_can_change_default_resolver():
    @strawberry.type
    class User:
        name: str

    @strawberry.type
    class Query:
        @strawberry.field
        def user(self) -> User:
            return {"name": "Patrick"}  # type: ignore

    schema = strawberry.Schema(
        query=Query,
        config=StrawberryConfig(default_resolver=getitem),
    )

    query = "{ user { name } }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data
    assert result.data["user"]["name"] == "Patrick"


def test_field_metadata():
    @strawberry.type
    class Query:
        a: str = strawberry.field(default="Example", metadata={"Foo": "Bar"})

    (a,) = dataclasses.fields(Query)
    assert a.metadata == {"Foo": "Bar"}


def test_field_type_priority():
    """Prioritise the field annotation on the class over the resolver annotation."""

    def my_resolver() -> str:
        return "1.33"

    @strawberry.type
    class Query:
        a: float = strawberry.field(resolver=my_resolver)

    schema = strawberry.Schema(Query)

    expected = """
    type Query {
      a: Float!
    }
    """

    assert print_schema(schema) == textwrap.dedent(expected).strip()

    query = "{ a }"

    result = schema.execute_sync(query, root_value=Query())

    assert not result.errors
    assert result.data == {
        "a": 1.33,
    }


def test_field_type_override():
    @strawberry.type
    class Query:
        a: float = strawberry.field(graphql_type=str)
        b = strawberry.field(graphql_type=int)

        @strawberry.field(graphql_type=float)
        def c(self):
            return "3.4"

    schema = strawberry.Schema(Query)

    expected = """
    type Query {
      a: String!
      b: Int!
      c: Float!
    }
    """

    assert print_schema(schema) == textwrap.dedent(expected).strip()

    query = "{ a, b, c }"

    result = schema.execute_sync(query, root_value=Query(a=1.33, b=2))

    assert not result.errors
    assert result.data == {
        "a": "1.33",
        "b": 2,
        "c": 3.4,
    }


def test_field_type_default():
    @strawberry.type
    class User:
        name: str = "James"

    @strawberry.type
    class Query:
        @strawberry.field
        def a(self) -> User:
            return User()

    schema = strawberry.Schema(Query)

    expected = """
    type Query {
      a: User!
    }

    type User {
      name: String!
    }
    """

    assert print_schema(schema) == textwrap.dedent(expected).strip()



import strawberry


# TODO: move this to common tests
def test_set_response_headers():
    from litestar import Litestar
    from litestar.testing import TestClient

    from strawberry.litestar import make_graphql_controller

    @strawberry.type
    class Query:
        @strawberry.field
        def abc(self, info: strawberry.Info) -> str:
            assert info.context.get("response") is not None
            info.context["response"].headers["X-Strawberry"] = "rocks"
            return "abc"

    schema = strawberry.Schema(query=Query)
    graphql_controller = make_graphql_controller(path="/graphql", schema=schema)
    app = Litestar(route_handlers=[graphql_controller])

    test_client = TestClient(app)
    response = test_client.post("/graphql", json={"query": "{ abc }"})

    assert response.status_code == 200
    assert response.json() == {"data": {"abc": "abc"}}

    assert response.headers["x-strawberry"] == "rocks"


def test_set_cookie_headers():
    from litestar import Litestar
    from litestar.testing import TestClient

    from strawberry.litestar import make_graphql_controller

    @strawberry.type
    class Query:
        @strawberry.field
        def abc(self, info: strawberry.Info) -> str:
            assert info.context.get("response") is not None
            info.context["response"].set_cookie(
                key="strawberry",
                value="rocks",
            )
            info.context["response"].set_cookie(
                key="Litestar",
                value="rocks",
            )
            return "abc"

    schema = strawberry.Schema(query=Query)
    graphql_controller = make_graphql_controller(path="/graphql", schema=schema)
    app = Litestar(route_handlers=[graphql_controller])

    test_client = TestClient(app)
    response = test_client.post("/graphql", json={"query": "{ abc }"})

    assert response.status_code == 200
    assert response.json() == {"data": {"abc": "abc"}}

    assert response.headers["set-cookie"] == (
        "strawberry=rocks; Path=/; SameSite=lax, Litestar=rocks; Path=/; SameSite=lax"
    )



from dataclasses import dataclass
from typing import Literal

ResultType = Literal[
    "error",
    "information",
    "note",
]


@dataclass
class Result:
    type: ResultType
    message: str
    line: int
    column: int



from typing import Annotated

import pytest

import strawberry
from strawberry import UNSET
from strawberry.exceptions import (
    InvalidArgumentTypeError,
    MultipleStrawberryArgumentsError,
)
from strawberry.types.base import StrawberryList, StrawberryOptional


def test_basic_arguments():
    @strawberry.type
    class Query:
        @strawberry.field
        def name(
            self, argument: str, optional_argument: str | None
        ) -> str:  # pragma: no cover
            return "Name"

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument1, argument2] = definition.fields[0].arguments

    assert argument1.python_name == "argument"
    assert argument1.graphql_name is None
    assert argument1.type is str

    assert argument2.python_name == "optional_argument"
    assert argument2.graphql_name is None
    assert isinstance(argument2.type, StrawberryOptional)
    assert argument2.type.of_type is str


def test_input_type_as_argument():
    @strawberry.input
    class Input:
        name: str

    @strawberry.type
    class Query:
        @strawberry.field
        def name(
            self, input: Input, optional_input: Input | None
        ) -> str:  # pragma: no cover
            return input.name

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument1, argument2] = definition.fields[0].arguments

    assert argument1.python_name == "input"
    assert argument1.graphql_name is None
    assert argument1.type is Input

    assert argument2.python_name == "optional_input"
    assert argument2.graphql_name is None
    assert isinstance(argument2.type, StrawberryOptional)
    assert argument2.type.of_type is Input


def test_arguments_lists():
    @strawberry.input
    class Input:
        name: str

    @strawberry.type
    class Query:
        @strawberry.field
        def names(self, inputs: list[Input]) -> list[str]:  # pragma: no cover
            return [input.name for input in inputs]

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument] = definition.fields[0].arguments

    assert argument.python_name == "inputs"
    assert argument.graphql_name is None
    assert isinstance(argument.type, StrawberryList)
    assert argument.type.of_type is Input


def test_arguments_lists_of_optionals():
    @strawberry.input
    class Input:
        name: str

    @strawberry.type
    class Query:
        @strawberry.field
        def names(self, inputs: list[Input | None]) -> list[str]:  # pragma: no cover
            return [input_.name for input_ in inputs if input_ is not None]

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument] = definition.fields[0].arguments

    assert argument.python_name == "inputs"
    assert argument.graphql_name is None
    assert isinstance(argument.type, StrawberryList)
    assert isinstance(argument.type.of_type, StrawberryOptional)
    assert argument.type.of_type.of_type is Input


def test_basic_arguments_on_resolver():
    def name_resolver(  # pragma: no cover
        id: strawberry.ID, argument: str, optional_argument: str | None
    ) -> str:
        return "Name"

    @strawberry.type
    class Query:
        name: str = strawberry.field(resolver=name_resolver)

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument1, argument2, argument3] = definition.fields[0].arguments

    assert argument1.python_name == "id"
    assert argument1.type is strawberry.ID

    assert argument2.python_name == "argument"
    assert argument2.type is str

    assert argument3.python_name == "optional_argument"
    assert isinstance(argument3.type, StrawberryOptional)
    assert argument3.type.of_type is str


def test_arguments_when_extending_a_type():
    def name_resolver(
        id: strawberry.ID, argument: str, optional_argument: str | None
    ) -> str:  # pragma: no cover
        return "Name"

    @strawberry.type
    class NameQuery:
        name: str = strawberry.field(resolver=name_resolver)

    @strawberry.type
    class Query(NameQuery):
        pass

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    assert len(definition.fields) == 1

    [argument1, argument2, argument3] = definition.fields[0].arguments

    assert argument1.python_name == "id"
    assert argument1.type is strawberry.ID

    assert argument2.python_name == "argument"
    assert argument2.type is str

    assert argument3.python_name == "optional_argument"
    assert isinstance(argument3.type, StrawberryOptional)
    assert argument3.type.of_type is str


def test_arguments_when_extending_multiple_types():
    def name_resolver(id: strawberry.ID) -> str:  # pragma: no cover
        return "Name"

    def name_2_resolver(id: strawberry.ID) -> str:  # pragma: no cover
        return "Name 2"

    @strawberry.type
    class NameQuery:
        name: str = strawberry.field(permission_classes=[], resolver=name_resolver)

    @strawberry.type
    class ExampleQuery:
        name_2: str = strawberry.field(permission_classes=[], resolver=name_2_resolver)

    @strawberry.type
    class RootQuery(NameQuery, ExampleQuery):
        pass

    definition = RootQuery.__strawberry_definition__

    assert definition.name == "RootQuery"

    assert len(definition.fields) == 2

    [argument1] = definition.fields[0].arguments

    assert argument1.python_name == "id"
    assert argument1.graphql_name is None
    assert argument1.type is strawberry.ID

    [argument2] = definition.fields[1].arguments

    assert argument2.python_name == "id"
    assert argument2.graphql_name is None
    assert argument2.type is strawberry.ID


def test_argument_with_default_value_none():
    @strawberry.type
    class Query:
        @strawberry.field
        def name(self, argument: str | None = None) -> str:  # pragma: no cover
            return "Name"

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument] = definition.fields[0].arguments

    assert argument.python_name == "argument"
    assert argument.graphql_name is None
    assert argument.default is None
    assert argument.description is None
    assert isinstance(argument.type, StrawberryOptional)
    assert argument.type.of_type is str


def test_argument_with_default_value_undefined():
    @strawberry.type
    class Query:
        @strawberry.field
        def name(self, argument: str | None) -> str:  # pragma: no cover
            return "Name"

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument] = definition.fields[0].arguments

    assert argument.python_name == "argument"
    assert argument.graphql_name is None
    assert argument.default is UNSET
    assert argument.description is None
    assert isinstance(argument.type, StrawberryOptional)
    assert argument.type.of_type is str


def test_annotated_argument_on_resolver():
    @strawberry.type
    class Query:
        @strawberry.field
        def name(  # type: ignore
            argument: Annotated[
                str,
                strawberry.argument(description="This is a description"),
            ],
        ) -> str:  # pragma: no cover
            return "Name"

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument] = definition.fields[0].arguments

    assert argument.python_name == "argument"
    assert argument.graphql_name is None
    assert argument.description == "This is a description"
    assert argument.type is str


def test_annotated_optional_arguments_on_resolver():
    @strawberry.type
    class Query:
        @strawberry.field
        def name(  # type: ignore
            argument: Annotated[
                str | None,
                strawberry.argument(description="This is a description"),
            ],
        ) -> str:  # pragma: no cover
            return "Name"

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument] = definition.fields[0].arguments

    assert argument.python_name == "argument"
    assert argument.graphql_name is None
    assert argument.description == "This is a description"
    assert isinstance(argument.type, StrawberryOptional)
    assert argument.type.of_type is str


def test_annotated_argument_with_default_value():
    @strawberry.type
    class Query:
        @strawberry.field
        def name(
            self,
            argument: Annotated[
                str,
                strawberry.argument(description="This is a description"),
            ] = "Patrick",
        ) -> str:  # pragma: no cover
            return "Name"

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument] = definition.fields[0].arguments

    assert argument.python_name == "argument"
    assert argument.graphql_name is None
    assert argument.description == "This is a description"
    assert argument.type is str
    assert argument.default == "Patrick"


def test_annotated_argument_with_rename():
    @strawberry.type
    class Query:
        @strawberry.field
        def name(
            self,
            arg: Annotated[
                str,
                strawberry.argument(name="argument"),
            ] = "Patrick",
        ) -> str:  # pragma: no cover
            return "Name"

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    assert len(definition.fields[0].arguments) == 1

    argument = definition.fields[0].arguments[0]

    assert argument.python_name == "arg"
    assert argument.graphql_name == "argument"
    assert argument.type is str
    assert argument.description is None
    assert argument.default == "Patrick"


def test_annotated_argument_with_graphql_type_override():
    BigInt = strawberry.scalar(int, name="BigInt", serialize=str, parse_value=int)

    @strawberry.type
    class Query:
        @strawberry.field
        def name(
            self,
            arg: Annotated[
                list[int],
                strawberry.argument(graphql_type=list[BigInt]),
            ],
        ) -> str:  # pragma: no cover
            return "Name"

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument] = definition.fields[0].arguments

    assert argument.python_name == "arg"
    assert isinstance(argument.type, StrawberryList)
    assert argument.type.of_type is BigInt


@pytest.mark.xfail(reason="Can't get field name from argument")
def test_multiple_annotated_arguments_exception():
    with pytest.raises(MultipleStrawberryArgumentsError) as error:

        @strawberry.field
        def name(
            argument: Annotated[
                str,
                strawberry.argument(description="This is a description"),
                strawberry.argument(description="Another description"),
            ],
        ) -> str:  # pragma: no cover
            return "Name"

    assert str(error.value) == (
        "Annotation for argument `argument` "
        "on field `name` cannot have multiple "
        "`strawberry.argument`s"
    )


def test_annotated_with_other_information():
    @strawberry.type
    class Query:
        @strawberry.field
        def name(
            self, argument: Annotated[str, "Some other info"]
        ) -> str:  # pragma: no cover
            return "Name"

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument] = definition.fields[0].arguments

    assert argument.python_name == "argument"
    assert argument.graphql_name is None
    assert argument.description is None
    assert argument.type is str


def test_annotated_python_39():
    from typing import Annotated

    @strawberry.type
    class Query:
        @strawberry.field
        def name(
            self,
            argument: Annotated[
                str,
                strawberry.argument(description="This is a description"),
            ],
        ) -> str:  # pragma: no cover
            return "Name"

    definition = Query.__strawberry_definition__

    assert definition.name == "Query"

    [argument] = definition.fields[0].arguments

    assert argument.python_name == "argument"
    assert argument.graphql_name is None
    assert argument.type is str
    assert argument.description == "This is a description"
    assert argument.type is str


@pytest.mark.raises_strawberry_exception(
    InvalidArgumentTypeError,
    'Argument "word" on field "add_word" cannot be of type "Union"',
)
def test_union_as_an_argument_type():
    @strawberry.type
    class Noun:
        text: str

    @strawberry.type
    class Verb:
        text: str

    Word = Annotated[Noun | Verb, strawberry.argument("Word")]

    @strawberry.field
    def add_word(word: Word) -> bool:
        return True


@pytest.mark.raises_strawberry_exception(
    InvalidArgumentTypeError,
    'Argument "adjective" on field "add_adjective" cannot be of type "Interface"',
)
def test_interface_as_an_argument_type():
    @strawberry.interface
    class Adjective:
        text: str

    @strawberry.field
    def add_adjective(adjective: Adjective) -> bool:
        return True


@pytest.mark.raises_strawberry_exception(
    InvalidArgumentTypeError,
    (
        'Argument "adjective" on field "add_adjective_resolver" cannot be '
        'of type "Interface"'
    ),
)
def test_resolver_with_invalid_field_argument_type():
    @strawberry.interface
    class Adjective:
        text: str

    def add_adjective_resolver(adjective: Adjective) -> bool:  # pragma: no cover
        return True

    @strawberry.type
    class Mutation:
        add_adjective: bool = strawberry.field(resolver=add_adjective_resolver)



# type: ignore
from typing import Any, Generic, NamedTuple, TypeVar

import pytest

import strawberry
from strawberry.exceptions import ConflictingArgumentsError
from strawberry.parent import Parent
from strawberry.types.info import Info


def test_resolver():
    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "I'm a resolver"

    schema = strawberry.Schema(query=Query)

    query = "{ hello }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["hello"] == "I'm a resolver"


@pytest.mark.asyncio
async def test_resolver_function():
    def function_resolver(root) -> str:
        return "I'm a function resolver"

    async def async_resolver(root) -> str:
        return "I'm an async resolver"

    def resolve_name(root) -> str:
        return root.name

    def resolve_say_hello(root, name: str) -> str:
        return f"Hello {name}"

    @strawberry.type
    class Query:
        hello: str = strawberry.field(resolver=function_resolver)
        hello_async: str = strawberry.field(resolver=async_resolver)
        get_name: str = strawberry.field(resolver=resolve_name)
        say_hello: str = strawberry.field(resolver=resolve_say_hello)

        name = "Patrick"

    schema = strawberry.Schema(query=Query)

    query = """{
        hello
        helloAsync
        getName
        sayHello(name: "Marco")
    }"""

    result = await schema.execute(query, root_value=Query())

    assert not result.errors
    assert result.data["hello"] == "I'm a function resolver"
    assert result.data["helloAsync"] == "I'm an async resolver"
    assert result.data["getName"] == "Patrick"
    assert result.data["sayHello"] == "Hello Marco"


def test_resolvers_on_types():
    def function_resolver(root) -> str:
        return "I'm a function resolver"

    def function_resolver_with_params(root, x: str) -> str:
        return f"I'm {x}"

    @strawberry.type
    class Example:
        hello: str = strawberry.field(resolver=function_resolver)
        hello_with_params: str = strawberry.field(
            resolver=function_resolver_with_params
        )

    @strawberry.type
    class Query:
        @strawberry.field
        def example(self) -> Example:
            return Example()

    schema = strawberry.Schema(query=Query)

    query = """{
        example {
            hello
            helloWithParams(x: "abc")
        }
    }"""

    result = schema.execute_sync(query, root_value=Query())

    assert not result.errors
    assert result.data["example"]["hello"] == "I'm a function resolver"
    assert result.data["example"]["helloWithParams"] == "I'm abc"


def test_optional_info_and_root_params_function_resolver():
    def function_resolver() -> str:
        return "I'm a function resolver"

    def function_resolver_with_root(root) -> str:
        return root._example

    def function_resolver_with_params(x: str) -> str:
        return f"I'm {x}"

    @strawberry.type
    class Query:
        hello: str = strawberry.field(resolver=function_resolver)
        hello_with_root: str = strawberry.field(resolver=function_resolver_with_root)
        hello_with_params: str = strawberry.field(
            resolver=function_resolver_with_params
        )

        def __post_init__(self):
            self._example = "Example"

    schema = strawberry.Schema(query=Query)

    query = """{
        hello
        helloWithRoot
        helloWithParams(x: "abc")
    }"""

    result = schema.execute_sync(query, root_value=Query())

    assert not result.errors
    assert result.data["hello"] == "I'm a function resolver"
    assert result.data["helloWithParams"] == "I'm abc"
    assert result.data["helloWithRoot"] == "Example"


def test_optional_info_and_root_params():
    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "I'm a function resolver"

        @strawberry.field
        def hello_with_params(self, x: str) -> str:
            return f"I'm {x}"

        @strawberry.field
        def uses_self(self) -> str:
            return f"I'm {self._example}"

        def __post_init__(self):
            self._example = "self"

    schema = strawberry.Schema(query=Query)

    query = """{
        hello
        helloWithParams(x: "abc")
        usesSelf
    }"""

    result = schema.execute_sync(query, root_value=Query())

    assert not result.errors
    assert result.data["hello"] == "I'm a function resolver"
    assert result.data["helloWithParams"] == "I'm abc"
    assert result.data["usesSelf"] == "I'm self"


def test_only_info_function_resolvers():
    def function_resolver(info: strawberry.Info) -> str:
        return f"I'm a function resolver for {info.field_name}"

    def function_resolver_with_params(info: strawberry.Info, x: str) -> str:
        return f"I'm {x} for {info.field_name}"

    @strawberry.type
    class Query:
        hello: str = strawberry.field(resolver=function_resolver)
        hello_with_params: str = strawberry.field(
            resolver=function_resolver_with_params
        )

    schema = strawberry.Schema(query=Query)

    query = """{
        hello
        helloWithParams(x: "abc")
    }"""

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["hello"] == "I'm a function resolver for hello"
    # TODO: in future, should we map names of info.field_name to the matching
    # dataclass field name?
    assert result.data["helloWithParams"] == "I'm abc for helloWithParams"


def test_classmethod_resolvers():
    global User

    @strawberry.type
    class User:
        name: str
        age: int

        @classmethod
        def get_users(cls) -> "list[User]":
            return [cls(name="Bob", age=10), cls(name="Nancy", age=30)]

    @strawberry.type
    class Query:
        users: list[User] = strawberry.field(resolver=User.get_users)

    schema = strawberry.Schema(query=Query)

    query = "{ users { name } }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"users": [{"name": "Bob"}, {"name": "Nancy"}]}

    del User


def test_staticmethod_resolvers():
    class Alphabet:
        @staticmethod
        def get_letters() -> list[str]:
            return ["a", "b", "c"]

    @strawberry.type
    class Query:
        letters: list[str] = strawberry.field(resolver=Alphabet.get_letters)

    schema = strawberry.Schema(query=Query)

    query = "{ letters }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"letters": ["a", "b", "c"]}


def test_lambda_resolvers():
    @strawberry.type
    class Query:
        letter: str = strawberry.field(resolver=lambda: "λ")

    schema = strawberry.Schema(query=Query)

    query = "{ letter }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"letter": "λ"}


def test_bounded_instance_method_resolvers():
    class CoolClass:
        def method(self):
            _ = self
            return "something"

    instance = CoolClass()

    @strawberry.type
    class Query:
        blah: str = strawberry.field(resolver=instance.method)

    schema = strawberry.Schema(query=Query)

    query = "{ blah }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"blah": "something"}


def test_extending_type():
    def name_resolver(id: strawberry.ID) -> str:
        return "Name"

    def name_2_resolver(id: strawberry.ID) -> str:
        return "Name 2"

    @strawberry.type
    class NameQuery:
        name: str = strawberry.field(permission_classes=[], resolver=name_resolver)

    @strawberry.type
    class ExampleQuery:
        name_2: str = strawberry.field(permission_classes=[], resolver=name_2_resolver)

    @strawberry.type
    class RootQuery(NameQuery, ExampleQuery):
        pass

    schema = strawberry.Schema(query=RootQuery)

    query = '{ name(id: "abc"), name2(id: "abc") }'

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"name": "Name", "name2": "Name 2"}


@pytest.mark.asyncio
async def test_async_list_resolver():
    @strawberry.type
    class Query:
        @strawberry.field
        async def best_flavours(self) -> list[str]:
            return ["strawberry", "pistachio"]

    schema = strawberry.Schema(query=Query)

    query = "{ bestFlavours }"

    result = await schema.execute(query, root_value=Query())

    assert not result.errors
    assert result.data["bestFlavours"] == ["strawberry", "pistachio"]


def test_can_use_source_as_argument_name():
    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self, source: str) -> str:
            return f"I'm a resolver for {source}"

    schema = strawberry.Schema(query=Query)

    query = '{ hello(source: "🍓") }'

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["hello"] == "I'm a resolver for 🍓"


def test_generic_resolver_factory():
    @strawberry.type
    class AType:
        some: int

    T = TypeVar("T")

    def resolver_factory(strawberry_type: type[T]):
        def resolver() -> T:
            return strawberry_type(some=1)

        return resolver

    @strawberry.type
    class Query:
        a_type: AType = strawberry.field(resolver_factory(AType))

    strawberry.Schema(query=Query)

    schema = strawberry.Schema(query=Query)

    query = "{ aType { some } }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"aType": {"some": 1}}


def test_generic_resolver_optional():
    @strawberry.type
    class AType:
        some: int

    T = TypeVar("T")

    def resolver() -> T | None:
        return AType(some=1)

    @strawberry.type
    class Query:
        a_type: AType | None = strawberry.field(resolver)

    strawberry.Schema(query=Query)

    schema = strawberry.Schema(query=Query)

    query = "{ aType { some } }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"aType": {"some": 1}}


def test_generic_resolver_container():
    T = TypeVar("T")

    @strawberry.type
    class Container(Generic[T]):
        item: T

    @strawberry.type
    class AType:
        some: int

    def resolver() -> Container[T]:
        return Container(item=AType(some=1))

    @strawberry.type
    class Query:
        a_type_in_container: Container[AType] = strawberry.field(resolver)

    strawberry.Schema(query=Query)

    schema = strawberry.Schema(query=Query)

    query = "{ aTypeInContainer { item { some } } }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"aTypeInContainer": {"item": {"some": 1}}}


def test_generic_resolver_union():
    T = TypeVar("T")

    @strawberry.type
    class AType:
        some: int

    @strawberry.type
    class OtherType:
        other: int

    def resolver() -> T | OtherType:
        return AType(some=1)

    @strawberry.type
    class Query:
        union_type: AType | OtherType = strawberry.field(resolver)

    strawberry.Schema(query=Query)

    schema = strawberry.Schema(query=Query)

    query = "{ unionType { ... on AType { some } } }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"unionType": {"some": 1}}


def test_generic_resolver_list():
    T = TypeVar("T")

    @strawberry.type
    class AType:
        some: int

    def resolver() -> list[T]:
        return [AType(some=1)]

    @strawberry.type
    class Query:
        list_type: list[AType] = strawberry.field(resolver)

    strawberry.Schema(query=Query)

    schema = strawberry.Schema(query=Query)

    query = "{ listType { some } }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"listType": [{"some": 1}]}


def type_based_info(info: strawberry.Info, icon: str) -> str:
    return f"I'm a resolver for {icon} {info.field_name}"


def generic_type_based_info(icon: str, info: strawberry.Info) -> str:
    return f"I'm a resolver for {icon} {info.field_name}"


def arbitrarily_named_info(icon: str, info_argument: Info) -> str:
    return f"I'm a resolver for {icon} {info_argument.field_name}"


@pytest.mark.parametrize(
    "resolver",
    [
        pytest.param(type_based_info),
        pytest.param(generic_type_based_info),
        pytest.param(arbitrarily_named_info),
    ],
)
def test_info_argument(resolver):
    @strawberry.type
    class ResolverGreeting:
        hello: str = strawberry.field(resolver=resolver)

    schema = strawberry.Schema(query=ResolverGreeting)
    result = schema.execute_sync('{ hello(icon: "🍓") }')

    assert not result.errors
    assert result.data["hello"] == "I'm a resolver for 🍓 hello"


class UserLiteral(NamedTuple):
    id: str


def parent_no_self(parent: Parent[UserLiteral]) -> str:
    return f"User {parent.id}"


class Foo:
    @staticmethod
    def static_method_parent(asdf: Parent[UserLiteral]) -> str:
        return f"User {asdf.id}"


@pytest.mark.parametrize(
    "resolver",
    [
        pytest.param(parent_no_self),
        pytest.param(Foo.static_method_parent),
    ],
)
def test_parent_argument(resolver):
    @strawberry.type
    class User:
        id: str
        name: str = strawberry.field(resolver=resolver)

    @strawberry.type
    class Query:
        @strawberry.field
        def user(self, user_id: str) -> User:
            return UserLiteral(user_id)

    schema = strawberry.Schema(query=Query)
    result = schema.execute_sync('{ user(userId: "🍓") { name } }')
    assert not result.errors
    assert result.data["user"]["name"] == "User 🍓"


def parent_and_self(self, parent: Parent[UserLiteral]) -> str:
    raise AssertionError("Unreachable code.")


def parent_self_and_root(self, root, parent: Parent[UserLiteral]) -> str:
    raise AssertionError("Unreachable code.")


def self_and_root(self, root) -> str:
    raise AssertionError("Unreachable code.")


def multiple_parents(user: Parent[Any], user2: Parent[Any]) -> str:
    raise AssertionError("Unreachable code.")


def multiple_infos(root, info1: Info, info2: Info) -> str:
    raise AssertionError("Unreachable code.")


@pytest.mark.parametrize(
    "resolver",
    [
        pytest.param(parent_self_and_root),
        pytest.param(multiple_parents),
        pytest.param(multiple_infos),
    ],
)
@pytest.mark.raises_strawberry_exception(
    ConflictingArgumentsError,
    match=(
        "Arguments .* define conflicting resources. "
        "Only one of these arguments may be defined per resolver."
    ),
)
def test_multiple_conflicting_reserved_arguments(resolver):
    @strawberry.type
    class Query:
        name: str = strawberry.field(resolver=resolver)

    strawberry.Schema(query=Query)


@pytest.mark.parametrize("resolver", [parent_and_self, self_and_root])
def test_self_should_not_raise_conflicting_arguments_error(resolver):
    @strawberry.type
    class Query:
        name: str = strawberry.field(resolver=resolver)

    strawberry.Schema(query=Query)



from __future__ import annotations

from typing import TYPE_CHECKING, Annotated

import strawberry

if TYPE_CHECKING:
    from tests.c import C


@strawberry.type
class D:
    id: strawberry.ID

    @strawberry.field
    async def c_list(
        self,
    ) -> list[Annotated[C, strawberry.lazy("tests.c")]]:  # pragma: no cover
        from tests.c import C

        return [C(id=self.id)]



import contextlib
from typing import Literal

import pytest
from inline_snapshot import snapshot

import strawberry
from strawberry.extensions.mask_errors import MaskErrors
from strawberry.schema.config import StrawberryConfig
from tests.conftest import skip_if_gql_32
from tests.http.clients.base import HttpClient
from tests.views.schema import Mutation, Query, Subscription

pytestmark = skip_if_gql_32("GraphQL 3.3.0 is required for incremental execution")


@pytest.mark.parametrize("method", ["get", "post"])
async def test_basic_defer(method: Literal["get", "post"], http_client: HttpClient):
    response = await http_client.query(
        method=method,
        query="""
        query HeroNameQuery {
            character {
                id
                ...NameFragment @defer
            }
        }
        fragment NameFragment on Hero {
            name
        }
        """,
    )

    async with contextlib.aclosing(response.streaming_json()) as stream:
        initial = await stream.__anext__()

        assert initial == snapshot(
            {
                "data": {"character": {"id": "1"}},
                "hasNext": True,
                "pending": [{"path": ["character"], "id": "0"}],
                # TODO: check if we need this and how to handle it
                "extensions": None,
            }
        )

        subsequent = await stream.__anext__()

        assert subsequent == snapshot(
            {
                "incremental": [
                    {
                        "data": {"name": "Thiago Bellini"},
                        "id": "0",
                        "path": ["character"],
                        "label": None,
                    }
                ],
                "completed": [{"id": "0"}],
                "hasNext": False,
                # TODO: same as above
                "extensions": None,
            }
        )


@pytest.mark.parametrize("method", ["get", "post"])
async def test_defer_with_mask_error_extension(
    method: Literal["get", "post"], incremental_http_client_class: type[HttpClient]
):
    schema = strawberry.Schema(
        query=Query,
        mutation=Mutation,
        subscription=Subscription,
        extensions=[MaskErrors()],
        config=StrawberryConfig(enable_experimental_incremental_execution=(True)),
    )

    http_client = incremental_http_client_class(schema=schema)

    response = await http_client.query(
        method=method,
        query="""
        query HeroNameQuery {
            someError
            character {
                id
                ...NameFragment @defer
            }
        }
        fragment NameFragment on Hero {
            name(fail: true)
        }
        """,
    )

    async with contextlib.aclosing(response.streaming_json()) as stream:
        initial_errors = (await stream.__anext__())["errors"]

        assert initial_errors[0]["message"] == "Unexpected error."

        # TODO: not yet supported properly (the error is not masked)
        subsequent = (await stream.__anext__())["completed"][0]
        subsequent_errors = subsequent["errors"]

        assert subsequent_errors[0]["message"] == "Failed to get name"



from __future__ import annotations

import contextlib
import json
from collections.abc import AsyncGenerator, Mapping, Sequence
from datetime import timedelta
from io import BytesIO
from typing import Any, Literal

from aiohttp import web
from aiohttp.client_ws import ClientWebSocketResponse
from aiohttp.http_websocket import WSMsgType
from aiohttp.test_utils import TestClient, TestServer

from strawberry.aiohttp.views import GraphQLView as BaseGraphQLView
from strawberry.http import GraphQLHTTPResponse
from strawberry.http.ides import GraphQL_IDE
from strawberry.schema import Schema
from strawberry.subscriptions import GRAPHQL_TRANSPORT_WS_PROTOCOL, GRAPHQL_WS_PROTOCOL
from strawberry.types import ExecutionResult
from tests.http.context import get_context
from tests.views.schema import Query
from tests.websockets.views import OnWSConnectMixin

from .base import (
    JSON,
    DebuggableGraphQLTransportWSHandler,
    DebuggableGraphQLWSHandler,
    HttpClient,
    Message,
    Response,
    ResultOverrideFunction,
    WebSocketClient,
)


class GraphQLView(OnWSConnectMixin, BaseGraphQLView[dict[str, object], object]):
    result_override: ResultOverrideFunction = None
    graphql_transport_ws_handler_class = DebuggableGraphQLTransportWSHandler
    graphql_ws_handler_class = DebuggableGraphQLWSHandler

    async def get_context(
        self, request: web.Request, response: web.Response | web.WebSocketResponse
    ) -> dict[str, object]:
        context = await super().get_context(request, response)

        return get_context(context)

    async def get_root_value(self, request: web.Request) -> Query:
        await super().get_root_value(request)  # for coverage
        return Query()

    async def process_result(
        self, request: web.Request, result: ExecutionResult
    ) -> GraphQLHTTPResponse:
        if self.result_override:
            return self.result_override(result)

        return await super().process_result(request, result)


class AioHttpClient(HttpClient):
    def __init__(
        self,
        schema: Schema,
        graphql_ide: GraphQL_IDE | None = "graphiql",
        allow_queries_via_get: bool = True,
        keep_alive: bool = False,
        keep_alive_interval: float = 1,
        subscription_protocols: Sequence[str] = (
            GRAPHQL_TRANSPORT_WS_PROTOCOL,
            GRAPHQL_WS_PROTOCOL,
        ),
        connection_init_wait_timeout: timedelta = timedelta(minutes=1),
        result_override: ResultOverrideFunction = None,
        multipart_uploads_enabled: bool = False,
        max_subscriptions_per_connection: int | None = 100,
    ):
        view = GraphQLView(
            schema=schema,
            graphql_ide=graphql_ide,
            allow_queries_via_get=allow_queries_via_get,
            keep_alive=keep_alive,
            keep_alive_interval=keep_alive_interval,
            subscription_protocols=subscription_protocols,
            connection_init_wait_timeout=connection_init_wait_timeout,
            multipart_uploads_enabled=multipart_uploads_enabled,
            max_subscriptions_per_connection=max_subscriptions_per_connection,
        )
        view.result_override = result_override

        self.app = web.Application()
        self.app.router.add_route("*", "/graphql", view)

    async def _graphql_request(
        self,
        method: Literal["get", "post"],
        query: str | None = None,
        operation_name: str | None = None,
        variables: dict[str, object] | None = None,
        files: dict[str, BytesIO] | None = None,
        headers: dict[str, str] | None = None,
        extensions: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> Response:
        async with TestClient(TestServer(self.app)) as client:
            body = self._build_body(
                query=query,
                operation_name=operation_name,
                variables=variables,
                files=files,
                method=method,
                extensions=extensions,
            )

            if body and files:
                body.update(files)

            if method == "get":
                kwargs["params"] = body
            else:
                kwargs["data"] = body if files else json.dumps(body)

            response = await getattr(client, method)(
                "/graphql",
                headers=self._get_headers(method=method, headers=headers, files=files),
                **kwargs,
            )

            return Response(
                status_code=response.status,
                data=(await response.text()).encode(),
                headers=response.headers,
            )

    async def request(
        self,
        url: str,
        method: Literal["head", "get", "post", "patch", "put", "delete"],
        headers: dict[str, str] | None = None,
    ) -> Response:
        async with TestClient(TestServer(self.app)) as client:
            response = await getattr(client, method)(url, headers=headers)

            return Response(
                status_code=response.status,
                data=(await response.text()).encode(),
                headers=response.headers,
            )

    async def get(
        self,
        url: str,
        headers: dict[str, str] | None = None,
    ) -> Response:
        return await self.request(url, "get", headers=headers)

    async def post(
        self,
        url: str,
        data: bytes | None = None,
        json: JSON | None = None,
        headers: dict[str, str] | None = None,
    ) -> Response:
        async with TestClient(TestServer(self.app)) as client:
            response = await client.post(
                "/graphql", headers=headers, data=data, json=json
            )

            return Response(
                status_code=response.status,
                data=(await response.text()).encode(),
                headers=dict(response.headers),
            )

    @contextlib.asynccontextmanager
    async def ws_connect(
        self,
        url: str,
        *,
        protocols: list[str],
    ) -> AsyncGenerator[WebSocketClient, None]:
        async with (
            TestClient(TestServer(self.app)) as client,
            client.ws_connect(url, protocols=protocols) as ws,
        ):
            yield AioWebSocketClient(ws)


class AioWebSocketClient(WebSocketClient):
    def __init__(self, ws: ClientWebSocketResponse):
        self.ws = ws
        self._reason: str | None = None

    async def send_text(self, payload: str) -> None:
        await self.ws.send_str(payload)

    async def send_json(self, payload: Mapping[str, object]) -> None:
        await self.ws.send_json(payload)

    async def send_bytes(self, payload: bytes) -> None:
        await self.ws.send_bytes(payload)

    async def receive(self, timeout: float | None = None) -> Message:
        m = await self.ws.receive(timeout)
        self._reason = m.extra
        return Message(type=m.type, data=m.data, extra=m.extra)

    async def receive_json(self, timeout: float | None = None) -> object:
        m = await self.ws.receive(timeout)
        assert m.type == WSMsgType.TEXT
        return json.loads(m.data)

    async def close(self) -> None:
        await self.ws.close()

    @property
    def accepted_subprotocol(self) -> str | None:
        return self.ws.protocol

    @property
    def closed(self) -> bool:
        return self.ws.closed

    @property
    def close_code(self) -> int:
        assert self.ws.close_code is not None
        return self.ws.close_code

    @property
    def close_reason(self) -> str | None:
        return self._reason
